@@ -1,10 +1,6 @@
-import platform
-import sys
 from pathlib import Path
 
 import pytest
-from twisted import version as twisted_version
-from twisted.python.versions import Version
 from twisted.web.http import H2_ENABLED
 
 from scrapy.utils.reactor import install_reactor
@@ -61,7 +57,7 @@ def pytest_addoption(parser):
 def reactor_pytest(request):
     if not request.cls:
         # doctests
-        return
+        return None
     request.cls.reactor_pytest = request.config.getoption("--reactor")
     return request.cls.reactor_pytest
 
@@ -85,14 +81,36 @@ def only_not_asyncio(request, reactor_pytest):
 def requires_uvloop(request):
     if not request.node.get_closest_marker("requires_uvloop"):
         return
-    if sys.implementation.name == "pypy":
-        pytest.skip("uvloop does not support pypy properly")
-    if platform.system() == "Windows":
-        pytest.skip("uvloop does not support Windows")
-    if twisted_version == Version("twisted", 21, 2, 0):
-        pytest.skip("https://twistedmatrix.com/trac/ticket/10106")
-    if sys.version_info >= (3, 12):
-        pytest.skip("uvloop doesn't support Python 3.12 yet")
+    try:
+        import uvloop
+
+        del uvloop
+    except ImportError:
+        pytest.skip("uvloop is not installed")
+
+
+@pytest.fixture(autouse=True)
+def requires_botocore(request):
+    if not request.node.get_closest_marker("requires_botocore"):
+        return
+    try:
+        import botocore
+
+        del botocore
+    except ImportError:
+        pytest.skip("botocore is not installed")
+
+
+@pytest.fixture(autouse=True)
+def requires_boto3(request):
+    if not request.node.get_closest_marker("requires_boto3"):
+        return
+    try:
+        import boto3
+
+        del boto3
+    except ImportError:
+        pytest.skip("boto3 is not installed")
 
 
 def pytest_configure(config):
