@@ -6,8 +6,6 @@ import pkgutil
 import sys
 import warnings
 
-from twisted import version as _txv
-
 # Declare top-level shortcuts
 from scrapy.http import FormRequest, Request
 from scrapy.item import Field, Item
@@ -17,7 +15,6 @@ from scrapy.spiders import Spider
 __all__ = [
     "__version__",
     "version_info",
-    "twisted_version",
     "Spider",
     "Request",
     "FormRequest",
@@ -30,13 +27,23 @@ __all__ = [
 # Scrapy and Twisted versions
 __version__ = (pkgutil.get_data(__package__, "VERSION") or b"").decode("ascii").strip()
 version_info = tuple(int(v) if v.isdigit() else v for v in __version__.split("."))
-twisted_version = (_txv.major, _txv.minor, _txv.micro)
 
 
-# Check minimum required Python version
-if sys.version_info < (3, 8):
-    print(f"Scrapy {__version__} requires Python 3.8+")
-    sys.exit(1)
+def __getattr__(name: str):
+    if name == "twisted_version":
+        import warnings
+
+        from twisted import version as _txv
+
+        from scrapy.exceptions import ScrapyDeprecationWarning
+
+        warnings.warn(
+            "The scrapy.twisted_version attribute is deprecated, use twisted.version instead",
+            ScrapyDeprecationWarning,
+        )
+        return _txv.major, _txv.minor, _txv.micro
+
+    raise AttributeError
 
 
 # Ignore noisy twisted deprecation warnings
