@@ -1,3 +1,6 @@
+from __future__ import annotations
+
+import argparse
 import os
 import re
 import string
@@ -11,7 +14,7 @@ from scrapy.commands import ScrapyCommand
 from scrapy.exceptions import UsageError
 from scrapy.utils.template import render_templatefile, string_camelcase
 
-TEMPLATES_TO_RENDER = (
+TEMPLATES_TO_RENDER: tuple[tuple[str, ...], ...] = (
     ("scrapy.cfg",),
     ("${project_name}", "settings.py.tmpl"),
     ("${project_name}", "items.py.tmpl"),
@@ -22,7 +25,7 @@ TEMPLATES_TO_RENDER = (
 IGNORE = ignore_patterns("*.pyc", "__pycache__", ".svn")
 
 
-def _make_writable(path):
+def _make_writable(path: str | os.PathLike) -> None:
     current_permissions = os.stat(path).st_mode
     os.chmod(path, current_permissions | OWNER_WRITE_PERMISSION)
 
@@ -31,14 +34,14 @@ class Command(ScrapyCommand):
     requires_project = False
     default_settings = {"LOG_ENABLED": False, "SPIDER_LOADER_WARN_ONLY": True}
 
-    def syntax(self):
+    def syntax(self) -> str:
         return "<project_name> [project_dir]"
 
-    def short_desc(self):
+    def short_desc(self) -> str:
         return "Create new project"
 
-    def _is_valid_name(self, project_name):
-        def _module_exists(module_name):
+    def _is_valid_name(self, project_name: str) -> bool:
+        def _module_exists(module_name: str) -> bool:
             spec = find_spec(module_name)
             return spec is not None and spec.loader is not None
 
@@ -53,7 +56,7 @@ class Command(ScrapyCommand):
             return True
         return False
 
-    def _copytree(self, src: Path, dst: Path):
+    def _copytree(self, src: Path, dst: Path) -> None:
         """
         Since the original function always creates the directory, to resolve
         the issue a new function had to be created. It's a simple copy and
@@ -84,7 +87,7 @@ class Command(ScrapyCommand):
         copystat(src, dst)
         _make_writable(dst)
 
-    def run(self, args, opts):
+    def run(self, args: list[str], opts: argparse.Namespace) -> None:
         if len(args) not in (1, 2):
             raise UsageError()
 
