@@ -28,8 +28,11 @@ In case of status 200 request, response.headers will come with two keys:
     'Size' - with size of the downloaded data
 """
 
+from __future__ import annotations
+
 import re
 from io import BytesIO
+from typing import TYPE_CHECKING, Any, BinaryIO
 from urllib.parse import unquote
 
 from twisted.internet.protocol import ClientCreator, Protocol
@@ -40,23 +43,37 @@ from scrapy.responsetypes import responsetypes
 from scrapy.utils.httpobj import urlparse_cached
 from scrapy.utils.python import to_bytes
 
+if TYPE_CHECKING:
+    from twisted.internet.defer import Deferred
+    from twisted.python.failure import Failure
+
+    # typing.Self requires Python 3.11
+    from typing_extensions import Self
+
+    from scrapy import Request, Spider
+    from scrapy.crawler import Crawler
+    from scrapy.settings import BaseSettings
+
 
 class ReceivedDataProtocol(Protocol):
-    def __init__(self, filename=None):
-        self.__filename = filename
-        self.body = open(filename, "wb") if filename else BytesIO()
-        self.size = 0
+    def __init__(self, filename: str | None = None):
+        self.__filename: str | None = filename
+        self.body: BinaryIO = open(filename, "wb") if filename else BytesIO()
+        self.size: int = 0
 
-    def dataReceived(self, data):
+    def dataReceived(self, data: bytes) -> None:
         self.body.write(data)
         self.size += len(data)
 
     @property
-    def filename(self):
+    def filename(self) -> str | None:
         return self.__filename
 
-    def close(self):
-        self.body.close() if self.filename else self.body.seek(0)
+    def close(self) -> None:
+        if self.filename:
+            self.body.close()
+        else:
+            self.body.seek(0)
 
 
 _CODE_RE = re.compile(r"\d+")
@@ -65,21 +82,21 @@ _CODE_RE = re.compile(r"\d+")
 class FTPDownloadHandler:
     lazy = False
 
-    CODE_MAPPING = {
+    CODE_MAPPING: dict[str, int] = {
         "550": 404,
         "default": 503,
     }
 
-    def __init__(self, settings):
+    def __init__(self, settings: BaseSettings):
         self.default_user = settings["FTP_USER"]
         self.default_password = settings["FTP_PASSWORD"]
         self.passive_mode = settings["FTP_PASSIVE_MODE"]
 
     @classmethod
-    def from_crawler(cls, crawler):
+    def from_crawler(cls, crawler: Crawler) -> Self:
         return cls(crawler.settings)
 
-    def download_request(self, request, spider):
+    def download_request(self, request: Request, spider: Spider) -> Deferred[Response]:
         from twisted.internet import reactor
 
         parsed_url = urlparse_cached(request)
@@ -91,28 +108,33 @@ class FTPDownloadHandler:
         creator = ClientCreator(
             reactor, FTPClient, user, password, passive=passive_mode
         )
-        dfd = creator.connectTCP(parsed_url.hostname, parsed_url.port or 21)
+        dfd: Deferred[FTPClient] = creator.connectTCP(
+            parsed_url.hostname, parsed_url.port or 21
+        )
         return dfd.addCallback(self.gotClient, request, unquote(parsed_url.path))
 
-    def gotClient(self, client, request, filepath):
+    def gotClient(
+        self, client: FTPClient, request: Request, filepath: str
+    ) -> Deferred[Response]:
         self.client = client
         protocol = ReceivedDataProtocol(request.meta.get("ftp_local_filename"))
-        return client.retrieveFile(filepath, protocol).addCallbacks(
-            callback=self._build_response,
-            callbackArgs=(request, protocol),
-            errback=self._failed,
-            errbackArgs=(request,),
-        )
+        d = client.retrieveFile(filepath, protocol)
+        d.addCallback(self._build_response, request, protocol)
+        d.addErrback(self._failed, request)
+        return d
 
-    def _build_response(self, result, request, protocol):
+    def _build_response(
+        self, result: Any, request: Request, protocol: ReceivedDataProtocol
+    ) -> Response:
         self.result = result
         protocol.close()
         headers = {"local filename": protocol.filename or "", "size": protocol.size}
         body = to_bytes(protocol.filename or protocol.body.read())
         respcls = responsetypes.from_args(url=request.url, body=body)
-        return respcls(url=request.url, status=200, body=body, headers=headers)
+        # hints for Headers-related types may need to be fixed to not use AnyStr
+        return respcls(url=request.url, status=200, body=body, headers=headers)  # type: ignore[arg-type]
 
-    def _failed(self, result, request):
+    def _failed(self, result: Failure, request: Request) -> Response:
         message = result.getErrorMessage()
         if result.type == CommandFailed:
             m = _CODE_RE.search(message)
@@ -122,4 +144,5 @@ class FTPDownloadHandler:
                 return Response(
                     url=request.url, status=httpcode, body=to_bytes(message)
                 )
+        assert result.type
         raise result.type(result.value)
