@@ -3,25 +3,34 @@ Downloader Middleware manager
 
 See documentation in docs/topics/downloader-middleware.rst
 """
-from typing import Any, Callable, Generator, List, Union, cast
+
+from __future__ import annotations
+
+from collections.abc import Callable
+from typing import TYPE_CHECKING, Any, cast
 
 from twisted.internet.defer import Deferred, inlineCallbacks
-from twisted.python.failure import Failure
 
-from scrapy import Spider
 from scrapy.exceptions import _InvalidOutput
 from scrapy.http import Request, Response
 from scrapy.middleware import MiddlewareManager
-from scrapy.settings import BaseSettings
 from scrapy.utils.conf import build_component_list
 from scrapy.utils.defer import deferred_from_coro, mustbe_deferred
 
+if TYPE_CHECKING:
+    from collections.abc import Generator
+
+    from twisted.python.failure import Failure
+
+    from scrapy import Spider
+    from scrapy.settings import BaseSettings
+
 
 class DownloaderMiddlewareManager(MiddlewareManager):
     component_name = "downloader middleware"
 
     @classmethod
-    def _get_mwlist_from_settings(cls, settings: BaseSettings) -> List[Any]:
+    def _get_mwlist_from_settings(cls, settings: BaseSettings) -> list[Any]:
         return build_component_list(settings.getwithbase("DOWNLOADER_MIDDLEWARES"))
 
     def _add_middleware(self, mw: Any) -> None:
@@ -33,10 +42,15 @@ class DownloaderMiddlewareManager(MiddlewareManager):
             self.methods["process_exception"].appendleft(mw.process_exception)
 
     def download(
-        self, download_func: Callable, request: Request, spider: Spider
-    ) -> Deferred:
+        self,
+        download_func: Callable[[Request, Spider], Deferred[Response]],
+        request: Request,
+        spider: Spider,
+    ) -> Deferred[Response | Request]:
         @inlineCallbacks
-        def process_request(request: Request) -> Generator[Deferred, Any, Any]:
+        def process_request(
+            request: Request,
+        ) -> Generator[Deferred[Any], Any, Response | Request]:
             for method in self.methods["process_request"]:
                 method = cast(Callable, method)
                 response = yield deferred_from_coro(
@@ -51,15 +65,15 @@ class DownloaderMiddlewareManager(MiddlewareManager):
                     )
                 if response:
                     return response
-            return (yield download_func(request=request, spider=spider))
+            return (yield download_func(request, spider))
 
         @inlineCallbacks
         def process_response(
-            response: Union[Response, Request]
-        ) -> Generator[Deferred, Any, Union[Response, Request]]:
+            response: Response | Request,
+        ) -> Generator[Deferred[Any], Any, Response | Request]:
             if response is None:
                 raise TypeError("Received None in process_response")
-            elif isinstance(response, Request):
+            if isinstance(response, Request):
                 return response
 
             for method in self.methods["process_response"]:
@@ -79,7 +93,7 @@ class DownloaderMiddlewareManager(MiddlewareManager):
         @inlineCallbacks
         def process_exception(
             failure: Failure,
-        ) -> Generator[Deferred, Any, Union[Failure, Response, Request]]:
+        ) -> Generator[Deferred[Any], Any, Failure | Response | Request]:
             exception = failure.value
             for method in self.methods["process_exception"]:
                 method = cast(Callable, method)
@@ -97,7 +111,9 @@ class DownloaderMiddlewareManager(MiddlewareManager):
                     return response
             return failure
 
-        deferred = mustbe_deferred(process_request, request)
+        deferred: Deferred[Response | Request] = mustbe_deferred(
+            process_request, request
+        )
         deferred.addErrback(process_exception)
         deferred.addCallback(process_response)
         return deferred
