@@ -3,27 +3,25 @@ from __future__ import annotations
 import logging
 import pprint
 import signal
-import warnings
-from typing import TYPE_CHECKING, Optional
-
-from twisted.internet import defer
-from zope.interface.exceptions import DoesNotImplement
-
-try:
-    # zope >= 5.0 only supports MultipleInvalid
-    from zope.interface.exceptions import MultipleInvalid
-except ImportError:
-    MultipleInvalid = None
+from typing import TYPE_CHECKING, Any, TypeVar, cast
 
+from twisted.internet.defer import (
+    Deferred,
+    DeferredList,
+    inlineCallbacks,
+    maybeDeferred,
+)
 from zope.interface.verify import verifyClass
 
 from scrapy import Spider, signals
+from scrapy.addons import AddonManager
 from scrapy.core.engine import ExecutionEngine
-from scrapy.exceptions import ScrapyDeprecationWarning
 from scrapy.extension import ExtensionManager
 from scrapy.interfaces import ISpiderLoader
-from scrapy.settings import Settings, overridden_settings
+from scrapy.logformatter import LogFormatter
+from scrapy.settings import BaseSettings, Settings, overridden_settings
 from scrapy.signalmanager import SignalManager
+from scrapy.statscollectors import StatsCollector
 from scrapy.utils.log import (
     LogCounterHandler,
     configure_logging,
@@ -32,7 +30,7 @@ from scrapy.utils.log import (
     log_reactor_info,
     log_scrapy_info,
 )
-from scrapy.utils.misc import create_instance, load_object
+from scrapy.utils.misc import build_from_crawler, load_object
 from scrapy.utils.ossignal import install_shutdown_handlers, signal_names
 from scrapy.utils.reactor import (
     install_reactor,
@@ -42,56 +40,79 @@ from scrapy.utils.reactor import (
 )
 
 if TYPE_CHECKING:
+    from collections.abc import Generator, Iterable
+
+    from scrapy.spiderloader import SpiderLoader
     from scrapy.utils.request import RequestFingerprinter
 
 
 logger = logging.getLogger(__name__)
 
+_T = TypeVar("_T")
+
 
 class Crawler:
-    def __init__(self, spidercls, settings=None, init_reactor: bool = False):
+    def __init__(
+        self,
+        spidercls: type[Spider],
+        settings: dict[str, Any] | Settings | None = None,
+        init_reactor: bool = False,
+    ):
         if isinstance(spidercls, Spider):
             raise ValueError("The spidercls argument must be a class, not an object")
 
         if isinstance(settings, dict) or settings is None:
             settings = Settings(settings)
 
-        self.spidercls = spidercls
-        self.settings = settings.copy()
+        self.spidercls: type[Spider] = spidercls
+        self.settings: Settings = settings.copy()
         self.spidercls.update_settings(self.settings)
+        self._update_root_log_handler()
 
-        self.signals = SignalManager(self)
+        self.addons: AddonManager = AddonManager(self)
+        self.signals: SignalManager = SignalManager(self)
 
+        self._init_reactor: bool = init_reactor
+        self.crawling: bool = False
+        self._started: bool = False
+
+        self.extensions: ExtensionManager | None = None
+        self.stats: StatsCollector | None = None
+        self.logformatter: LogFormatter | None = None
+        self.request_fingerprinter: RequestFingerprinter | None = None
+        self.spider: Spider | None = None
+        self.engine: ExecutionEngine | None = None
+
+    def _update_root_log_handler(self) -> None:
+        if get_scrapy_root_handler() is not None:
+            # scrapy root handler already installed: update it with new settings
+            install_scrapy_root_handler(self.settings)
+
+    def _apply_settings(self) -> None:
+        if self.settings.frozen:
+            return
+
+        self.addons.load_settings(self.settings)
         self.stats = load_object(self.settings["STATS_CLASS"])(self)
 
         handler = LogCounterHandler(self, level=self.settings.get("LOG_LEVEL"))
         logging.root.addHandler(handler)
-
-        d = dict(overridden_settings(self.settings))
-        logger.info(
-            "Overridden settings:\n%(settings)s", {"settings": pprint.pformat(d)}
-        )
-
-        if get_scrapy_root_handler() is not None:
-            # scrapy root handler already installed: update it with new settings
-            install_scrapy_root_handler(self.settings)
         # lambda is assigned to Crawler attribute because this way it is not
-        # garbage collected after leaving __init__ scope
+        # garbage collected after leaving the scope
         self.__remove_handler = lambda: logging.root.removeHandler(handler)
         self.signals.connect(self.__remove_handler, signals.engine_stopped)
 
-        lf_cls = load_object(self.settings["LOG_FORMATTER"])
+        lf_cls: type[LogFormatter] = load_object(self.settings["LOG_FORMATTER"])
         self.logformatter = lf_cls.from_crawler(self)
 
-        self.request_fingerprinter: RequestFingerprinter = create_instance(
+        self.request_fingerprinter = build_from_crawler(
             load_object(self.settings["REQUEST_FINGERPRINTER_CLASS"]),
-            settings=self.settings,
-            crawler=self,
+            self,
         )
 
-        reactor_class = self.settings["TWISTED_REACTOR"]
-        event_loop = self.settings["ASYNCIO_EVENT_LOOP"]
-        if init_reactor:
+        reactor_class: str = self.settings["TWISTED_REACTOR"]
+        event_loop: str = self.settings["ASYNCIO_EVENT_LOOP"]
+        if self._init_reactor:
             # this needs to be done after the spider settings are merged,
             # but before something imports twisted.internet.reactor
             if reactor_class:
@@ -104,44 +125,140 @@ class Crawler:
             if is_asyncio_reactor_installed() and event_loop:
                 verify_installed_asyncio_event_loop(event_loop)
 
+            log_reactor_info()
+
         self.extensions = ExtensionManager.from_crawler(self)
-
         self.settings.freeze()
-        self.crawling = False
-        self.spider = None
-        self.engine: Optional[ExecutionEngine] = None
 
-    @defer.inlineCallbacks
-    def crawl(self, *args, **kwargs):
+        d = dict(overridden_settings(self.settings))
+        logger.info(
+            "Overridden settings:\n%(settings)s", {"settings": pprint.pformat(d)}
+        )
+
+    @inlineCallbacks
+    def crawl(self, *args: Any, **kwargs: Any) -> Generator[Deferred[Any], Any, None]:
         if self.crawling:
             raise RuntimeError("Crawling already taking place")
-        self.crawling = True
+        if self._started:
+            raise RuntimeError(
+                "Cannot run Crawler.crawl() more than once on the same instance."
+            )
+        self.crawling = self._started = True
 
         try:
             self.spider = self._create_spider(*args, **kwargs)
+            self._apply_settings()
+            self._update_root_log_handler()
             self.engine = self._create_engine()
             start_requests = iter(self.spider.start_requests())
             yield self.engine.open_spider(self.spider, start_requests)
-            yield defer.maybeDeferred(self.engine.start)
+            yield maybeDeferred(self.engine.start)
         except Exception:
             self.crawling = False
             if self.engine is not None:
                 yield self.engine.close()
             raise
 
-    def _create_spider(self, *args, **kwargs):
+    def _create_spider(self, *args: Any, **kwargs: Any) -> Spider:
         return self.spidercls.from_crawler(self, *args, **kwargs)
 
-    def _create_engine(self):
+    def _create_engine(self) -> ExecutionEngine:
         return ExecutionEngine(self, lambda _: self.stop())
 
-    @defer.inlineCallbacks
-    def stop(self):
+    @inlineCallbacks
+    def stop(self) -> Generator[Deferred[Any], Any, None]:
         """Starts a graceful stop of the crawler and returns a deferred that is
         fired when the crawler is stopped."""
         if self.crawling:
             self.crawling = False
-            yield defer.maybeDeferred(self.engine.stop)
+            assert self.engine
+            yield maybeDeferred(self.engine.stop)
+
+    @staticmethod
+    def _get_component(
+        component_class: type[_T], components: Iterable[Any]
+    ) -> _T | None:
+        for component in components:
+            if isinstance(component, component_class):
+                return component
+        return None
+
+    def get_addon(self, cls: type[_T]) -> _T | None:
+        """Return the run-time instance of an :ref:`add-on <topics-addons>` of
+        the specified class or a subclass, or ``None`` if none is found.
+
+        .. versionadded:: 2.12
+        """
+        return self._get_component(cls, self.addons.addons)
+
+    def get_downloader_middleware(self, cls: type[_T]) -> _T | None:
+        """Return the run-time instance of a :ref:`downloader middleware
+        <topics-downloader-middleware>` of the specified class or a subclass,
+        or ``None`` if none is found.
+
+        .. versionadded:: 2.12
+
+        This method can only be called after the crawl engine has been created,
+        e.g. at signals :signal:`engine_started` or :signal:`spider_opened`.
+        """
+        if not self.engine:
+            raise RuntimeError(
+                "Crawler.get_downloader_middleware() can only be called after "
+                "the crawl engine has been created."
+            )
+        return self._get_component(cls, self.engine.downloader.middleware.middlewares)
+
+    def get_extension(self, cls: type[_T]) -> _T | None:
+        """Return the run-time instance of an :ref:`extension
+        <topics-extensions>` of the specified class or a subclass,
+        or ``None`` if none is found.
+
+        .. versionadded:: 2.12
+
+        This method can only be called after the extension manager has been
+        created, e.g. at signals :signal:`engine_started` or
+        :signal:`spider_opened`.
+        """
+        if not self.extensions:
+            raise RuntimeError(
+                "Crawler.get_extension() can only be called after the "
+                "extension manager has been created."
+            )
+        return self._get_component(cls, self.extensions.middlewares)
+
+    def get_item_pipeline(self, cls: type[_T]) -> _T | None:
+        """Return the run-time instance of a :ref:`item pipeline
+        <topics-item-pipeline>` of the specified class or a subclass, or
+        ``None`` if none is found.
+
+        .. versionadded:: 2.12
+
+        This method can only be called after the crawl engine has been created,
+        e.g. at signals :signal:`engine_started` or :signal:`spider_opened`.
+        """
+        if not self.engine:
+            raise RuntimeError(
+                "Crawler.get_item_pipeline() can only be called after the "
+                "crawl engine has been created."
+            )
+        return self._get_component(cls, self.engine.scraper.itemproc.middlewares)
+
+    def get_spider_middleware(self, cls: type[_T]) -> _T | None:
+        """Return the run-time instance of a :ref:`spider middleware
+        <topics-spider-middleware>` of the specified class or a subclass, or
+        ``None`` if none is found.
+
+        .. versionadded:: 2.12
+
+        This method can only be called after the crawl engine has been created,
+        e.g. at signals :signal:`engine_started` or :signal:`spider_opened`.
+        """
+        if not self.engine:
+            raise RuntimeError(
+                "Crawler.get_spider_middleware() can only be called after the "
+                "crawl engine has been created."
+            )
+        return self._get_component(cls, self.engine.scraper.spidermw.middlewares)
 
 
 class CrawlerRunner:
@@ -164,45 +281,28 @@ class CrawlerRunner:
     )
 
     @staticmethod
-    def _get_spider_loader(settings):
+    def _get_spider_loader(settings: BaseSettings) -> SpiderLoader:
         """Get SpiderLoader instance from settings"""
         cls_path = settings.get("SPIDER_LOADER_CLASS")
         loader_cls = load_object(cls_path)
-        excs = (
-            (DoesNotImplement, MultipleInvalid) if MultipleInvalid else DoesNotImplement
-        )
-        try:
-            verifyClass(ISpiderLoader, loader_cls)
-        except excs:
-            warnings.warn(
-                "SPIDER_LOADER_CLASS (previously named SPIDER_MANAGER_CLASS) does "
-                "not fully implement scrapy.interfaces.ISpiderLoader interface. "
-                "Please add all missing methods to avoid unexpected runtime errors.",
-                category=ScrapyDeprecationWarning,
-                stacklevel=2,
-            )
-        return loader_cls.from_settings(settings.frozencopy())
+        verifyClass(ISpiderLoader, loader_cls)
+        return cast("SpiderLoader", loader_cls.from_settings(settings.frozencopy()))
 
-    def __init__(self, settings=None):
+    def __init__(self, settings: dict[str, Any] | Settings | None = None):
         if isinstance(settings, dict) or settings is None:
             settings = Settings(settings)
-        self.settings = settings
-        self.spider_loader = self._get_spider_loader(settings)
-        self._crawlers = set()
-        self._active = set()
+        self.settings: Settings = settings
+        self.spider_loader: SpiderLoader = self._get_spider_loader(settings)
+        self._crawlers: set[Crawler] = set()
+        self._active: set[Deferred[None]] = set()
         self.bootstrap_failed = False
 
-    @property
-    def spiders(self):
-        warnings.warn(
-            "CrawlerRunner.spiders attribute is renamed to "
-            "CrawlerRunner.spider_loader.",
-            category=ScrapyDeprecationWarning,
-            stacklevel=2,
-        )
-        return self.spider_loader
-
-    def crawl(self, crawler_or_spidercls, *args, **kwargs):
+    def crawl(
+        self,
+        crawler_or_spidercls: type[Spider] | str | Crawler,
+        *args: Any,
+        **kwargs: Any,
+    ) -> Deferred[None]:
         """
         Run a crawler with the provided arguments.
 
@@ -232,12 +332,12 @@ class CrawlerRunner:
         crawler = self.create_crawler(crawler_or_spidercls)
         return self._crawl(crawler, *args, **kwargs)
 
-    def _crawl(self, crawler, *args, **kwargs):
+    def _crawl(self, crawler: Crawler, *args: Any, **kwargs: Any) -> Deferred[None]:
         self.crawlers.add(crawler)
         d = crawler.crawl(*args, **kwargs)
         self._active.add(d)
 
-        def _done(result):
+        def _done(result: _T) -> _T:
             self.crawlers.discard(crawler)
             self._active.discard(d)
             self.bootstrap_failed |= not getattr(crawler, "spider", None)
@@ -245,7 +345,9 @@ class CrawlerRunner:
 
         return d.addBoth(_done)
 
-    def create_crawler(self, crawler_or_spidercls):
+    def create_crawler(
+        self, crawler_or_spidercls: type[Spider] | str | Crawler
+    ) -> Crawler:
         """
         Return a :class:`~scrapy.crawler.Crawler` object.
 
@@ -265,21 +367,21 @@ class CrawlerRunner:
             return crawler_or_spidercls
         return self._create_crawler(crawler_or_spidercls)
 
-    def _create_crawler(self, spidercls):
+    def _create_crawler(self, spidercls: str | type[Spider]) -> Crawler:
         if isinstance(spidercls, str):
             spidercls = self.spider_loader.load(spidercls)
         return Crawler(spidercls, self.settings)
 
-    def stop(self):
+    def stop(self) -> Deferred[Any]:
         """
         Stops simultaneously all the crawling jobs taking place.
 
         Returns a deferred that is fired when they all have ended.
         """
-        return defer.DeferredList([c.stop() for c in list(self.crawlers)])
+        return DeferredList([c.stop() for c in list(self.crawlers)])
 
-    @defer.inlineCallbacks
-    def join(self):
+    @inlineCallbacks
+    def join(self) -> Generator[Deferred[Any], Any, None]:
         """
         join()
 
@@ -287,7 +389,7 @@ class CrawlerRunner:
         completed their executions.
         """
         while self._active:
-            yield defer.DeferredList(self._active)
+            yield DeferredList(self._active)
 
 
 class CrawlerProcess(CrawlerRunner):
@@ -314,13 +416,17 @@ class CrawlerProcess(CrawlerRunner):
     process. See :ref:`run-from-script` for an example.
     """
 
-    def __init__(self, settings=None, install_root_handler=True):
+    def __init__(
+        self,
+        settings: dict[str, Any] | Settings | None = None,
+        install_root_handler: bool = True,
+    ):
         super().__init__(settings)
         configure_logging(self.settings, install_root_handler)
         log_scrapy_info(self.settings)
-        self._initialized_reactor = False
+        self._initialized_reactor: bool = False
 
-    def _signal_shutdown(self, signum, _):
+    def _signal_shutdown(self, signum: int, _: Any) -> None:
         from twisted.internet import reactor
 
         install_shutdown_handlers(self._signal_kill)
@@ -331,7 +437,7 @@ class CrawlerProcess(CrawlerRunner):
         )
         reactor.callFromThread(self._graceful_stop_reactor)
 
-    def _signal_kill(self, signum, _):
+    def _signal_kill(self, signum: int, _: Any) -> None:
         from twisted.internet import reactor
 
         install_shutdown_handlers(signal.SIG_IGN)
@@ -341,14 +447,17 @@ class CrawlerProcess(CrawlerRunner):
         )
         reactor.callFromThread(self._stop_reactor)
 
-    def _create_crawler(self, spidercls):
+    def _create_crawler(self, spidercls: type[Spider] | str) -> Crawler:
         if isinstance(spidercls, str):
             spidercls = self.spider_loader.load(spidercls)
         init_reactor = not self._initialized_reactor
         self._initialized_reactor = True
+        # temporary cast until self.spider_loader is typed
         return Crawler(spidercls, self.settings, init_reactor=init_reactor)
 
-    def start(self, stop_after_crawl=True, install_signal_handlers=True):
+    def start(
+        self, stop_after_crawl: bool = True, install_signal_handlers: bool = True
+    ) -> None:
         """
         This method starts a :mod:`~twisted.internet.reactor`, adjusts its pool
         size to :setting:`REACTOR_THREADPOOL_MAXSIZE`, and installs a DNS cache
@@ -360,8 +469,8 @@ class CrawlerProcess(CrawlerRunner):
         :param bool stop_after_crawl: stop or not the reactor when all
             crawlers have finished
 
-        :param bool install_signal_handlers: whether to install the shutdown
-            handlers (default: True)
+        :param bool install_signal_handlers: whether to install the OS signal
+            handlers from Twisted and Scrapy (default: True)
         """
         from twisted.internet import reactor
 
@@ -372,22 +481,26 @@ class CrawlerProcess(CrawlerRunner):
                 return
             d.addBoth(self._stop_reactor)
 
-        if install_signal_handlers:
-            install_shutdown_handlers(self._signal_shutdown)
         resolver_class = load_object(self.settings["DNS_RESOLVER"])
-        resolver = create_instance(resolver_class, self.settings, self, reactor=reactor)
+        # We pass self, which is CrawlerProcess, instead of Crawler here,
+        # which works because the default resolvers only use crawler.settings.
+        resolver = build_from_crawler(resolver_class, self, reactor=reactor)  # type: ignore[arg-type]
         resolver.install_on_reactor()
         tp = reactor.getThreadPool()
         tp.adjustPoolsize(maxthreads=self.settings.getint("REACTOR_THREADPOOL_MAXSIZE"))
         reactor.addSystemEventTrigger("before", "shutdown", self.stop)
-        reactor.run(installSignalHandlers=False)  # blocking call
+        if install_signal_handlers:
+            reactor.addSystemEventTrigger(
+                "after", "startup", install_shutdown_handlers, self._signal_shutdown
+            )
+        reactor.run(installSignalHandlers=install_signal_handlers)  # blocking call
 
-    def _graceful_stop_reactor(self):
+    def _graceful_stop_reactor(self) -> Deferred[Any]:
         d = self.stop()
         d.addBoth(self._stop_reactor)
         return d
 
-    def _stop_reactor(self, _=None):
+    def _stop_reactor(self, _: Any = None) -> None:
         from twisted.internet import reactor
 
         try:
