@@ -3,83 +3,149 @@ Files Pipeline
 
 See documentation in topics/media-pipeline.rst
 """
+
+from __future__ import annotations
+
 import base64
 import functools
 import hashlib
 import logging
 import mimetypes
-import os
 import time
+import warnings
 from collections import defaultdict
 from contextlib import suppress
 from ftplib import FTP
 from io import BytesIO
-from os import PathLike
 from pathlib import Path
-from typing import DefaultDict, Optional, Set, Union
+from typing import IO, TYPE_CHECKING, Any, NoReturn, Protocol, TypedDict, cast
 from urllib.parse import urlparse
 
 from itemadapter import ItemAdapter
-from twisted.internet import defer, threads
+from twisted.internet.defer import Deferred, maybeDeferred
+from twisted.internet.threads import deferToThread
 
-from scrapy.exceptions import IgnoreRequest, NotConfigured
-from scrapy.http import Request
+from scrapy.exceptions import IgnoreRequest, NotConfigured, ScrapyDeprecationWarning
+from scrapy.http import Request, Response
 from scrapy.http.request import NO_CALLBACK
-from scrapy.pipelines.media import MediaPipeline
-from scrapy.settings import Settings
+from scrapy.pipelines.media import FileInfo, FileInfoOrError, MediaPipeline
+from scrapy.settings import BaseSettings, Settings
 from scrapy.utils.boto import is_botocore_available
 from scrapy.utils.datatypes import CaseInsensitiveDict
+from scrapy.utils.deprecate import method_is_overridden
 from scrapy.utils.ftp import ftp_store_file
 from scrapy.utils.log import failure_to_exc_info
-from scrapy.utils.misc import md5sum
-from scrapy.utils.python import to_bytes
+from scrapy.utils.python import get_func_args, global_object_name, to_bytes
 from scrapy.utils.request import referer_str
 
+if TYPE_CHECKING:
+    from collections.abc import Callable
+    from os import PathLike
+
+    from twisted.python.failure import Failure
+
+    # typing.Self requires Python 3.11
+    from typing_extensions import Self
+
+    from scrapy import Spider
+    from scrapy.crawler import Crawler
+
+
 logger = logging.getLogger(__name__)
 
 
-def _to_string(path: Union[str, PathLike]) -> str:
+def _to_string(path: str | PathLike[str]) -> str:
     return str(path)  # convert a Path object to string
 
 
+def _md5sum(file: IO[bytes]) -> str:
+    """Calculate the md5 checksum of a file-like object without reading its
+    whole content in memory.
+
+    >>> from io import BytesIO
+    >>> _md5sum(BytesIO(b'file content to hash'))
+    '784406af91dd5a54fbb9c84c2236595a'
+    """
+    m = hashlib.md5()  # nosec
+    while True:
+        d = file.read(8096)
+        if not d:
+            break
+        m.update(d)
+    return m.hexdigest()
+
+
 class FileException(Exception):
     """General media error exception"""
 
 
+class StatInfo(TypedDict, total=False):
+    checksum: str
+    last_modified: float
+
+
+class FilesStoreProtocol(Protocol):
+    def __init__(self, basedir: str): ...
+
+    def persist_file(
+        self,
+        path: str,
+        buf: BytesIO,
+        info: MediaPipeline.SpiderInfo,
+        meta: dict[str, Any] | None = None,
+        headers: dict[str, str] | None = None,
+    ) -> Deferred[Any] | None: ...
+
+    def stat_file(
+        self, path: str, info: MediaPipeline.SpiderInfo
+    ) -> StatInfo | Deferred[StatInfo]: ...
+
+
 class FSFilesStore:
-    def __init__(self, basedir: Union[str, PathLike]):
+    def __init__(self, basedir: str | PathLike[str]):
         basedir = _to_string(basedir)
         if "://" in basedir:
             basedir = basedir.split("://", 1)[1]
-        self.basedir = basedir
+        self.basedir: str = basedir
         self._mkdir(Path(self.basedir))
-        self.created_directories: DefaultDict[str, Set[str]] = defaultdict(set)
+        self.created_directories: defaultdict[MediaPipeline.SpiderInfo, set[str]] = (
+            defaultdict(set)
+        )
 
     def persist_file(
-        self, path: Union[str, PathLike], buf, info, meta=None, headers=None
-    ):
+        self,
+        path: str | PathLike[str],
+        buf: BytesIO,
+        info: MediaPipeline.SpiderInfo,
+        meta: dict[str, Any] | None = None,
+        headers: dict[str, str] | None = None,
+    ) -> None:
         absolute_path = self._get_filesystem_path(path)
         self._mkdir(absolute_path.parent, info)
         absolute_path.write_bytes(buf.getvalue())
 
-    def stat_file(self, path: Union[str, PathLike], info):
+    def stat_file(
+        self, path: str | PathLike[str], info: MediaPipeline.SpiderInfo
+    ) -> StatInfo:
         absolute_path = self._get_filesystem_path(path)
         try:
             last_modified = absolute_path.stat().st_mtime
-        except os.error:
+        except OSError:
             return {}
 
         with absolute_path.open("rb") as f:
-            checksum = md5sum(f)
+            checksum = _md5sum(f)
 
         return {"last_modified": last_modified, "checksum": checksum}
 
-    def _get_filesystem_path(self, path: Union[str, PathLike]) -> Path:
+    def _get_filesystem_path(self, path: str | PathLike[str]) -> Path:
         path_comps = _to_string(path).split("/")
         return Path(self.basedir, *path_comps)
 
-    def _mkdir(self, dirname: Path, domain: Optional[str] = None):
-        seen = self.created_directories[domain] if domain else set()
+    def _mkdir(
+        self, dirname: Path, domain: MediaPipeline.SpiderInfo | None = None
+    ) -> None:
+        seen: set[str] = self.created_directories[domain] if domain else set()
         if str(dirname) not in seen:
             if not dirname.exists():
                 dirname.mkdir(parents=True)
@@ -100,7 +166,7 @@ class S3FilesStore:
         "Cache-Control": "max-age=172800",
     }
 
-    def __init__(self, uri):
+    def __init__(self, uri: str):
         if not is_botocore_available():
             raise NotConfigured("missing botocore library")
         import botocore.session
@@ -120,8 +186,10 @@ class S3FilesStore:
             raise ValueError(f"Incorrect URI scheme in {uri}, expected 's3'")
         self.bucket, self.prefix = uri[5:].split("/", 1)
 
-    def stat_file(self, path, info):
-        def _onsuccess(boto_key):
+    def stat_file(
+        self, path: str, info: MediaPipeline.SpiderInfo
+    ) -> Deferred[StatInfo]:
+        def _onsuccess(boto_key: dict[str, Any]) -> StatInfo:
             checksum = boto_key["ETag"].strip('"')
             last_modified = boto_key["LastModified"]
             modified_stamp = time.mktime(last_modified.timetuple())
@@ -129,21 +197,31 @@ class S3FilesStore:
 
         return self._get_boto_key(path).addCallback(_onsuccess)
 
-    def _get_boto_key(self, path):
+    def _get_boto_key(self, path: str) -> Deferred[dict[str, Any]]:
         key_name = f"{self.prefix}{path}"
-        return threads.deferToThread(
-            self.s3_client.head_object, Bucket=self.bucket, Key=key_name
+        return cast(
+            "Deferred[dict[str, Any]]",
+            deferToThread(
+                self.s3_client.head_object, Bucket=self.bucket, Key=key_name  # type: ignore[attr-defined]
+            ),
         )
 
-    def persist_file(self, path, buf, info, meta=None, headers=None):
+    def persist_file(
+        self,
+        path: str,
+        buf: BytesIO,
+        info: MediaPipeline.SpiderInfo,
+        meta: dict[str, Any] | None = None,
+        headers: dict[str, str] | None = None,
+    ) -> Deferred[Any]:
         """Upload file to S3 storage"""
         key_name = f"{self.prefix}{path}"
         buf.seek(0)
         extra = self._headers_to_botocore_kwargs(self.HEADERS)
         if headers:
             extra.update(self._headers_to_botocore_kwargs(headers))
-        return threads.deferToThread(
-            self.s3_client.put_object,
+        return deferToThread(
+            self.s3_client.put_object,  # type: ignore[attr-defined]
             Bucket=self.bucket,
             Key=key_name,
             Body=buf,
@@ -152,7 +230,7 @@ class S3FilesStore:
             **extra,
         )
 
-    def _headers_to_botocore_kwargs(self, headers):
+    def _headers_to_botocore_kwargs(self, headers: dict[str, Any]) -> dict[str, Any]:
         """Convert headers to botocore keyword arguments."""
         # This is required while we need to support both boto and botocore.
         mapping = CaseInsensitiveDict(
@@ -184,14 +262,13 @@ class S3FilesStore:
                 "X-Amz-Website-Redirect-Location": "WebsiteRedirectLocation",
             }
         )
-        extra = {}
+        extra: dict[str, Any] = {}
         for key, value in headers.items():
             try:
                 kwarg = mapping[key]
             except KeyError:
                 raise TypeError(f'Header "{key}" is not supported by botocore')
-            else:
-                extra[kwarg] = value
+            extra[kwarg] = value
         return extra
 
 
@@ -204,13 +281,13 @@ class GCSFilesStore:
     # Overridden from settings.FILES_STORE_GCS_ACL in FilesPipeline.from_settings.
     POLICY = None
 
-    def __init__(self, uri):
+    def __init__(self, uri: str):
         from google.cloud import storage
 
         client = storage.Client(project=self.GCS_PROJECT_ID)
         bucket, prefix = uri[5:].split("/", 1)
         self.bucket = client.bucket(bucket)
-        self.prefix = prefix
+        self.prefix: str = prefix
         permissions = self.bucket.test_iam_permissions(
             ["storage.objects.get", "storage.objects.create"]
         )
@@ -226,8 +303,10 @@ class GCSFilesStore:
                 {"bucket": bucket},
             )
 
-    def stat_file(self, path, info):
-        def _onsuccess(blob):
+    def stat_file(
+        self, path: str, info: MediaPipeline.SpiderInfo
+    ) -> Deferred[StatInfo]:
+        def _onsuccess(blob) -> StatInfo:
             if blob:
                 checksum = base64.b64decode(blob.md5_hash).hex()
                 last_modified = time.mktime(blob.updated.timetuple())
@@ -235,24 +314,32 @@ class GCSFilesStore:
             return {}
 
         blob_path = self._get_blob_path(path)
-        return threads.deferToThread(self.bucket.get_blob, blob_path).addCallback(
-            _onsuccess
+        return cast(
+            Deferred[StatInfo],
+            deferToThread(self.bucket.get_blob, blob_path).addCallback(_onsuccess),
         )
 
-    def _get_content_type(self, headers):
+    def _get_content_type(self, headers: dict[str, str] | None) -> str:
         if headers and "Content-Type" in headers:
             return headers["Content-Type"]
         return "application/octet-stream"
 
-    def _get_blob_path(self, path):
+    def _get_blob_path(self, path: str) -> str:
         return self.prefix + path
 
-    def persist_file(self, path, buf, info, meta=None, headers=None):
+    def persist_file(
+        self,
+        path: str,
+        buf: BytesIO,
+        info: MediaPipeline.SpiderInfo,
+        meta: dict[str, Any] | None = None,
+        headers: dict[str, str] | None = None,
+    ) -> Deferred[Any]:
         blob_path = self._get_blob_path(path)
         blob = self.bucket.blob(blob_path)
         blob.cache_control = self.CACHE_CONTROL
         blob.metadata = {k: str(v) for k, v in (meta or {}).items()}
-        return threads.deferToThread(
+        return deferToThread(
             blob.upload_from_string,
             data=buf.getvalue(),
             content_type=self._get_content_type(headers),
@@ -261,24 +348,35 @@ class GCSFilesStore:
 
 
 class FTPFilesStore:
-    FTP_USERNAME = None
-    FTP_PASSWORD = None
-    USE_ACTIVE_MODE = None
+    FTP_USERNAME: str | None = None
+    FTP_PASSWORD: str | None = None
+    USE_ACTIVE_MODE: bool | None = None
 
-    def __init__(self, uri):
+    def __init__(self, uri: str):
         if not uri.startswith("ftp://"):
             raise ValueError(f"Incorrect URI scheme in {uri}, expected 'ftp'")
         u = urlparse(uri)
-        self.port = u.port
-        self.host = u.hostname
+        assert u.port
+        assert u.hostname
+        self.port: int = u.port
+        self.host: str = u.hostname
         self.port = int(u.port or 21)
-        self.username = u.username or self.FTP_USERNAME
-        self.password = u.password or self.FTP_PASSWORD
-        self.basedir = u.path.rstrip("/")
+        assert self.FTP_USERNAME
+        assert self.FTP_PASSWORD
+        self.username: str = u.username or self.FTP_USERNAME
+        self.password: str = u.password or self.FTP_PASSWORD
+        self.basedir: str = u.path.rstrip("/")
 
-    def persist_file(self, path, buf, info, meta=None, headers=None):
+    def persist_file(
+        self,
+        path: str,
+        buf: BytesIO,
+        info: MediaPipeline.SpiderInfo,
+        meta: dict[str, Any] | None = None,
+        headers: dict[str, str] | None = None,
+    ) -> Deferred[Any]:
         path = f"{self.basedir}/{path}"
-        return threads.deferToThread(
+        return deferToThread(
             ftp_store_file,
             path=path,
             file=buf,
@@ -289,8 +387,10 @@ class FTPFilesStore:
             use_active_mode=self.USE_ACTIVE_MODE,
         )
 
-    def stat_file(self, path, info):
-        def _stat_file(path):
+    def stat_file(
+        self, path: str, info: MediaPipeline.SpiderInfo
+    ) -> Deferred[StatInfo]:
+        def _stat_file(path: str) -> StatInfo:
             try:
                 ftp = FTP()
                 ftp.connect(self.host, self.port)
@@ -299,14 +399,14 @@ class FTPFilesStore:
                     ftp.set_pasv(False)
                 file_path = f"{self.basedir}/{path}"
                 last_modified = float(ftp.voidcmd(f"MDTM {file_path}")[4:].strip())
-                m = hashlib.md5()
+                m = hashlib.md5()  # nosec
                 ftp.retrbinary(f"RETR {file_path}", m.update)
                 return {"last_modified": last_modified, "checksum": m.hexdigest()}
             # The file doesn't exist
             except Exception:
                 return {}
 
-        return threads.deferToThread(_stat_file, path)
+        return cast("Deferred[StatInfo]", deferToThread(_stat_file, path))
 
 
 class FilesPipeline(MediaPipeline):
@@ -328,47 +428,108 @@ class FilesPipeline(MediaPipeline):
 
     """
 
-    MEDIA_NAME = "file"
-    EXPIRES = 90
-    STORE_SCHEMES = {
+    MEDIA_NAME: str = "file"
+    EXPIRES: int = 90
+    STORE_SCHEMES: dict[str, type[FilesStoreProtocol]] = {
         "": FSFilesStore,
         "file": FSFilesStore,
         "s3": S3FilesStore,
         "gs": GCSFilesStore,
         "ftp": FTPFilesStore,
     }
-    DEFAULT_FILES_URLS_FIELD = "file_urls"
-    DEFAULT_FILES_RESULT_FIELD = "files"
+    DEFAULT_FILES_URLS_FIELD: str = "file_urls"
+    DEFAULT_FILES_RESULT_FIELD: str = "files"
 
-    def __init__(self, store_uri, download_func=None, settings=None):
+    def __init__(
+        self,
+        store_uri: str | PathLike[str],
+        download_func: Callable[[Request, Spider], Response] | None = None,
+        settings: Settings | dict[str, Any] | None = None,
+        *,
+        crawler: Crawler | None = None,
+    ):
         store_uri = _to_string(store_uri)
         if not store_uri:
             raise NotConfigured
 
-        if isinstance(settings, dict) or settings is None:
+        if crawler is not None:
+            if settings is not None:
+                warnings.warn(
+                    f"FilesPipeline.__init__() was called with a crawler instance and a settings instance"
+                    f" when creating {global_object_name(self.__class__)}. The settings instance will be ignored"
+                    f" and crawler.settings will be used. The settings argument will be removed in a future Scrapy version.",
+                    category=ScrapyDeprecationWarning,
+                    stacklevel=2,
+                )
+            settings = crawler.settings
+        elif isinstance(settings, dict) or settings is None:
             settings = Settings(settings)
         cls_name = "FilesPipeline"
-        self.store = self._get_store(store_uri)
+        self.store: FilesStoreProtocol = self._get_store(store_uri)
         resolve = functools.partial(
             self._key_for_pipe, base_class_name=cls_name, settings=settings
         )
-        self.expires = settings.getint(resolve("FILES_EXPIRES"), self.EXPIRES)
+        self.expires: int = settings.getint(resolve("FILES_EXPIRES"), self.EXPIRES)
         if not hasattr(self, "FILES_URLS_FIELD"):
             self.FILES_URLS_FIELD = self.DEFAULT_FILES_URLS_FIELD
         if not hasattr(self, "FILES_RESULT_FIELD"):
             self.FILES_RESULT_FIELD = self.DEFAULT_FILES_RESULT_FIELD
-        self.files_urls_field = settings.get(
+        self.files_urls_field: str = settings.get(
             resolve("FILES_URLS_FIELD"), self.FILES_URLS_FIELD
         )
-        self.files_result_field = settings.get(
+        self.files_result_field: str = settings.get(
             resolve("FILES_RESULT_FIELD"), self.FILES_RESULT_FIELD
         )
 
-        super().__init__(download_func=download_func, settings=settings)
+        super().__init__(
+            download_func=download_func,
+            settings=settings if not crawler else None,
+            crawler=crawler,
+        )
 
     @classmethod
-    def from_settings(cls, settings):
-        s3store = cls.STORE_SCHEMES["s3"]
+    def from_settings(cls, settings: Settings) -> Self:
+        warnings.warn(
+            f"{cls.__name__}.from_settings() is deprecated, use from_crawler() instead.",
+            category=ScrapyDeprecationWarning,
+            stacklevel=2,
+        )
+        return cls._from_settings(settings, None)
+
+    @classmethod
+    def from_crawler(cls, crawler: Crawler) -> Self:
+        if method_is_overridden(cls, FilesPipeline, "from_settings"):
+            warnings.warn(
+                f"{global_object_name(cls)} overrides FilesPipeline.from_settings()."
+                f" This method is deprecated and won't be called in future Scrapy versions,"
+                f" please update your code so that it overrides from_crawler() instead.",
+                category=ScrapyDeprecationWarning,
+            )
+            o = cls.from_settings(crawler.settings)
+            o._finish_init(crawler)
+            return o
+        return cls._from_settings(crawler.settings, crawler)
+
+    @classmethod
+    def _from_settings(cls, settings: Settings, crawler: Crawler | None) -> Self:
+        cls._update_stores(settings)
+        store_uri = settings["FILES_STORE"]
+        if "crawler" in get_func_args(cls.__init__):
+            o = cls(store_uri, crawler=crawler)
+        else:
+            o = cls(store_uri, settings=settings)
+            if crawler:
+                o._finish_init(crawler)
+            warnings.warn(
+                f"{global_object_name(cls)}.__init__() doesn't take a crawler argument."
+                " This is deprecated and the argument will be required in future Scrapy versions.",
+                category=ScrapyDeprecationWarning,
+            )
+        return o
+
+    @classmethod
+    def _update_stores(cls, settings: BaseSettings) -> None:
+        s3store: type[S3FilesStore] = cast(type[S3FilesStore], cls.STORE_SCHEMES["s3"])
         s3store.AWS_ACCESS_KEY_ID = settings["AWS_ACCESS_KEY_ID"]
         s3store.AWS_SECRET_ACCESS_KEY = settings["AWS_SECRET_ACCESS_KEY"]
         s3store.AWS_SESSION_TOKEN = settings["AWS_SESSION_TOKEN"]
@@ -378,19 +539,20 @@ class FilesPipeline(MediaPipeline):
         s3store.AWS_VERIFY = settings["AWS_VERIFY"]
         s3store.POLICY = settings["FILES_STORE_S3_ACL"]
 
-        gcs_store = cls.STORE_SCHEMES["gs"]
+        gcs_store: type[GCSFilesStore] = cast(
+            type[GCSFilesStore], cls.STORE_SCHEMES["gs"]
+        )
         gcs_store.GCS_PROJECT_ID = settings["GCS_PROJECT_ID"]
         gcs_store.POLICY = settings["FILES_STORE_GCS_ACL"] or None
 
-        ftp_store = cls.STORE_SCHEMES["ftp"]
+        ftp_store: type[FTPFilesStore] = cast(
+            type[FTPFilesStore], cls.STORE_SCHEMES["ftp"]
+        )
         ftp_store.FTP_USERNAME = settings["FTP_USER"]
         ftp_store.FTP_PASSWORD = settings["FTP_PASSWORD"]
         ftp_store.USE_ACTIVE_MODE = settings.getbool("FEED_STORAGE_FTP_ACTIVE")
 
-        store_uri = settings["FILES_STORE"]
-        return cls(store_uri, settings=settings)
-
-    def _get_store(self, uri: str):
+    def _get_store(self, uri: str) -> FilesStoreProtocol:
         if Path(uri).is_absolute():  # to support win32 paths like: C:\\some\dir
             scheme = "file"
         else:
@@ -398,19 +560,21 @@ class FilesPipeline(MediaPipeline):
         store_cls = self.STORE_SCHEMES[scheme]
         return store_cls(uri)
 
-    def media_to_download(self, request, info, *, item=None):
-        def _onsuccess(result):
+    def media_to_download(
+        self, request: Request, info: MediaPipeline.SpiderInfo, *, item: Any = None
+    ) -> Deferred[FileInfo | None]:
+        def _onsuccess(result: StatInfo) -> FileInfo | None:
             if not result:
-                return  # returning None force download
+                return None  # returning None force download
 
             last_modified = result.get("last_modified", None)
             if not last_modified:
-                return  # returning None force download
+                return None  # returning None force download
 
             age_seconds = time.time() - last_modified
             age_days = age_seconds / 60 / 60 / 24
             if age_days > self.expires:
-                return  # returning None force download
+                return None  # returning None force download
 
             referer = referer_str(request)
             logger.debug(
@@ -430,18 +594,22 @@ class FilesPipeline(MediaPipeline):
             }
 
         path = self.file_path(request, info=info, item=item)
-        dfd = defer.maybeDeferred(self.store.stat_file, path, info)
-        dfd.addCallbacks(_onsuccess, lambda _: None)
-        dfd.addErrback(
+        # maybeDeferred() overloads don't seem to support a Union[_T, Deferred[_T]] return type
+        dfd: Deferred[StatInfo] = maybeDeferred(self.store.stat_file, path, info)  # type: ignore[call-overload]
+        dfd2: Deferred[FileInfo | None] = dfd.addCallback(_onsuccess)
+        dfd2.addErrback(lambda _: None)
+        dfd2.addErrback(
             lambda f: logger.error(
                 self.__class__.__name__ + ".store.stat_file",
                 exc_info=failure_to_exc_info(f),
                 extra={"spider": info.spider},
             )
         )
-        return dfd
+        return dfd2
 
-    def media_failed(self, failure, request, info):
+    def media_failed(
+        self, failure: Failure, request: Request, info: MediaPipeline.SpiderInfo
+    ) -> NoReturn:
         if not isinstance(failure.value, IgnoreRequest):
             referer = referer_str(request)
             logger.warning(
@@ -458,7 +626,14 @@ class FilesPipeline(MediaPipeline):
 
         raise FileException
 
-    def media_downloaded(self, response, request, info, *, item=None):
+    def media_downloaded(
+        self,
+        response: Response,
+        request: Request,
+        info: MediaPipeline.SpiderInfo,
+        *,
+        item: Any = None,
+    ) -> FileInfo:
         referer = referer_str(request)
 
         if response.status != 200:
@@ -517,30 +692,49 @@ class FilesPipeline(MediaPipeline):
             "status": status,
         }
 
-    def inc_stats(self, spider, status):
+    def inc_stats(self, spider: Spider, status: str) -> None:
+        assert spider.crawler.stats
         spider.crawler.stats.inc_value("file_count", spider=spider)
         spider.crawler.stats.inc_value(f"file_status_count/{status}", spider=spider)
 
     # Overridable Interface
-    def get_media_requests(self, item, info):
+    def get_media_requests(
+        self, item: Any, info: MediaPipeline.SpiderInfo
+    ) -> list[Request]:
         urls = ItemAdapter(item).get(self.files_urls_field, [])
         return [Request(u, callback=NO_CALLBACK) for u in urls]
 
-    def file_downloaded(self, response, request, info, *, item=None):
+    def file_downloaded(
+        self,
+        response: Response,
+        request: Request,
+        info: MediaPipeline.SpiderInfo,
+        *,
+        item: Any = None,
+    ) -> str:
         path = self.file_path(request, response=response, info=info, item=item)
         buf = BytesIO(response.body)
-        checksum = md5sum(buf)
+        checksum = _md5sum(buf)
         buf.seek(0)
         self.store.persist_file(path, buf, info)
         return checksum
 
-    def item_completed(self, results, item, info):
+    def item_completed(
+        self, results: list[FileInfoOrError], item: Any, info: MediaPipeline.SpiderInfo
+    ) -> Any:
         with suppress(KeyError):
             ItemAdapter(item)[self.files_result_field] = [x for ok, x in results if ok]
         return item
 
-    def file_path(self, request, response=None, info=None, *, item=None):
-        media_guid = hashlib.sha1(to_bytes(request.url)).hexdigest()
+    def file_path(
+        self,
+        request: Request,
+        response: Response | None = None,
+        info: MediaPipeline.SpiderInfo | None = None,
+        *,
+        item: Any = None,
+    ) -> str:
+        media_guid = hashlib.sha1(to_bytes(request.url)).hexdigest()  # nosec
         media_ext = Path(request.url).suffix
         # Handles empty and wild extensions by trying to guess the
         # mime type then extension or default to empty string otherwise
@@ -548,5 +742,5 @@ class FilesPipeline(MediaPipeline):
             media_ext = ""
             media_type = mimetypes.guess_type(request.url)[0]
             if media_type:
-                media_ext = mimetypes.guess_extension(media_type)
+                media_ext = cast(str, mimetypes.guess_extension(media_type))
         return f"full/{media_guid}{media_ext}"
