@@ -1,10 +1,14 @@
-from typing import Any, List, Tuple
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, Any
 
 from pydispatch import dispatcher
-from twisted.internet.defer import Deferred
 
 from scrapy.utils import signal as _signal
 
+if TYPE_CHECKING:
+    from twisted.internet.defer import Deferred
+
 
 class SignalManager:
     def __init__(self, sender: Any = dispatcher.Anonymous):
@@ -36,7 +40,7 @@ class SignalManager:
         kwargs.setdefault("sender", self.sender)
         dispatcher.disconnect(receiver, signal, **kwargs)
 
-    def send_catch_log(self, signal: Any, **kwargs: Any) -> List[Tuple[Any, Any]]:
+    def send_catch_log(self, signal: Any, **kwargs: Any) -> list[tuple[Any, Any]]:
         """
         Send a signal, catch exceptions and log them.
 
@@ -46,7 +50,9 @@ class SignalManager:
         kwargs.setdefault("sender", self.sender)
         return _signal.send_catch_log(signal, **kwargs)
 
-    def send_catch_log_deferred(self, signal: Any, **kwargs: Any) -> Deferred:
+    def send_catch_log_deferred(
+        self, signal: Any, **kwargs: Any
+    ) -> Deferred[list[tuple[Any, Any]]]:
         """
         Like :meth:`send_catch_log` but supports returning
         :class:`~twisted.internet.defer.Deferred` objects from signal handlers.
