@@ -3,21 +3,23 @@ from __future__ import annotations
 import traceback
 import warnings
 from collections import defaultdict
-from types import ModuleType
-from typing import TYPE_CHECKING, DefaultDict, Dict, List, Tuple, Type
+from typing import TYPE_CHECKING
 
 from zope.interface import implementer
 
-from scrapy import Request, Spider
 from scrapy.interfaces import ISpiderLoader
-from scrapy.settings import BaseSettings
 from scrapy.utils.misc import walk_modules
 from scrapy.utils.spider import iter_spider_classes
 
 if TYPE_CHECKING:
+    from types import ModuleType
+
     # typing.Self requires Python 3.11
     from typing_extensions import Self
 
+    from scrapy import Request, Spider
+    from scrapy.settings import BaseSettings
+
 
 @implementer(ISpiderLoader)
 class SpiderLoader:
@@ -27,10 +29,10 @@ class SpiderLoader:
     """
 
     def __init__(self, settings: BaseSettings):
-        self.spider_modules: List[str] = settings.getlist("SPIDER_MODULES")
+        self.spider_modules: list[str] = settings.getlist("SPIDER_MODULES")
         self.warn_only: bool = settings.getbool("SPIDER_LOADER_WARN_ONLY")
-        self._spiders: Dict[str, Type[Spider]] = {}
-        self._found: DefaultDict[str, List[Tuple[str, str]]] = defaultdict(list)
+        self._spiders: dict[str, type[Spider]] = {}
+        self._found: defaultdict[str, list[tuple[str, str]]] = defaultdict(list)
         self._load_all_spiders()
 
     def _check_name_duplicates(self) -> None:
@@ -62,7 +64,7 @@ class SpiderLoader:
             try:
                 for module in walk_modules(name):
                     self._load_spiders(module)
-            except ImportError:
+            except (ImportError, SyntaxError):
                 if self.warn_only:
                     warnings.warn(
                         f"\n{traceback.format_exc()}Could not load spiders "
@@ -78,7 +80,7 @@ class SpiderLoader:
     def from_settings(cls, settings: BaseSettings) -> Self:
         return cls(settings)
 
-    def load(self, spider_name: str) -> Type[Spider]:
+    def load(self, spider_name: str) -> type[Spider]:
         """
         Return the Spider class for the given spider name. If the spider
         name is not found, raise a KeyError.
@@ -88,7 +90,7 @@ class SpiderLoader:
         except KeyError:
             raise KeyError(f"Spider not found: {spider_name}")
 
-    def find_by_request(self, request: Request) -> List[str]:
+    def find_by_request(self, request: Request) -> list[str]:
         """
         Return the list of spider names that can handle the given request.
         """
@@ -96,7 +98,7 @@ class SpiderLoader:
             name for name, cls in self._spiders.items() if cls.handles_request(request)
         ]
 
-    def list(self) -> List[str]:
+    def list(self) -> list[str]:
         """
         Return a list with the names of all spiders available in the project.
         """
