@@ -1,17 +1,33 @@
+from __future__ import annotations
+
+from collections.abc import Callable
 from functools import wraps
+from typing import TYPE_CHECKING, Any
+
+if TYPE_CHECKING:
+    from collections.abc import Iterable
+
+EmbedFuncT = Callable[..., None]
+KnownShellsT = dict[str, Callable[..., EmbedFuncT]]
 
 
-def _embed_ipython_shell(namespace={}, banner=""):
+def _embed_ipython_shell(
+    namespace: dict[str, Any] = {}, banner: str = ""
+) -> EmbedFuncT:
     """Start an IPython Shell"""
     try:
-        from IPython.terminal.embed import InteractiveShellEmbed
+        from IPython.terminal.embed import InteractiveShellEmbed  # noqa: T100
         from IPython.terminal.ipapp import load_default_config
     except ImportError:
-        from IPython.frontend.terminal.embed import InteractiveShellEmbed
-        from IPython.frontend.terminal.ipapp import load_default_config
+        from IPython.frontend.terminal.embed import (  # type: ignore[no-redef]  # noqa: T100
+            InteractiveShellEmbed,
+        )
+        from IPython.frontend.terminal.ipapp import (  # type: ignore[no-redef]
+            load_default_config,
+        )
 
     @wraps(_embed_ipython_shell)
-    def wrapper(namespace=namespace, banner=""):
+    def wrapper(namespace: dict[str, Any] = namespace, banner: str = "") -> None:
         config = load_default_config()
         # Always use .instance() to ensure _instance propagation to all parents
         # this is needed for <TAB> completion works well for new imports
@@ -26,30 +42,36 @@ def _embed_ipython_shell(namespace={}, banner=""):
     return wrapper
 
 
-def _embed_bpython_shell(namespace={}, banner=""):
+def _embed_bpython_shell(
+    namespace: dict[str, Any] = {}, banner: str = ""
+) -> EmbedFuncT:
     """Start a bpython shell"""
     import bpython
 
     @wraps(_embed_bpython_shell)
-    def wrapper(namespace=namespace, banner=""):
+    def wrapper(namespace: dict[str, Any] = namespace, banner: str = "") -> None:
         bpython.embed(locals_=namespace, banner=banner)
 
     return wrapper
 
 
-def _embed_ptpython_shell(namespace={}, banner=""):
+def _embed_ptpython_shell(
+    namespace: dict[str, Any] = {}, banner: str = ""
+) -> EmbedFuncT:
     """Start a ptpython shell"""
     import ptpython.repl
 
     @wraps(_embed_ptpython_shell)
-    def wrapper(namespace=namespace, banner=""):
+    def wrapper(namespace: dict[str, Any] = namespace, banner: str = "") -> None:
         print(banner)
         ptpython.repl.embed(locals=namespace)
 
     return wrapper
 
 
-def _embed_standard_shell(namespace={}, banner=""):
+def _embed_standard_shell(
+    namespace: dict[str, Any] = {}, banner: str = ""
+) -> EmbedFuncT:
     """Start a standard python shell"""
     import code
 
@@ -60,16 +82,16 @@ def _embed_standard_shell(namespace={}, banner=""):
     else:
         import rlcompleter  # noqa: F401
 
-        readline.parse_and_bind("tab:complete")
+        readline.parse_and_bind("tab:complete")  # type: ignore[attr-defined]
 
     @wraps(_embed_standard_shell)
-    def wrapper(namespace=namespace, banner=""):
+    def wrapper(namespace: dict[str, Any] = namespace, banner: str = "") -> None:
         code.interact(banner=banner, local=namespace)
 
     return wrapper
 
 
-DEFAULT_PYTHON_SHELLS = {
+DEFAULT_PYTHON_SHELLS: KnownShellsT = {
     "ptpython": _embed_ptpython_shell,
     "ipython": _embed_ipython_shell,
     "bpython": _embed_bpython_shell,
@@ -77,7 +99,9 @@ DEFAULT_PYTHON_SHELLS = {
 }
 
 
-def get_shell_embed_func(shells=None, known_shells=None):
+def get_shell_embed_func(
+    shells: Iterable[str] | None = None, known_shells: KnownShellsT | None = None
+) -> EmbedFuncT | None:
     """Return the first acceptable shell-embed function
     from a given list of shell names.
     """
@@ -93,9 +117,14 @@ def get_shell_embed_func(shells=None, known_shells=None):
                 return known_shells[shell]()
             except ImportError:
                 continue
+    return None
 
 
-def start_python_console(namespace=None, banner="", shells=None):
+def start_python_console(
+    namespace: dict[str, Any] | None = None,
+    banner: str = "",
+    shells: Iterable[str] | None = None,
+) -> None:
     """Start Python console bound to the given namespace.
     Readline support and tab completion will be used on Unix, if available.
     """
