@@ -1,6 +1,10 @@
 """Helper functions for working with signals"""
-import collections.abc
+
+from __future__ import annotations
+
 import logging
+from collections.abc import Sequence
+from typing import Any as TypingAny
 
 from pydispatch.dispatcher import (
     Anonymous,
@@ -20,20 +24,22 @@ from scrapy.utils.log import failure_to_exc_info
 logger = logging.getLogger(__name__)
 
 
-def send_catch_log(signal=Any, sender=Anonymous, *arguments, **named):
+def send_catch_log(
+    signal: TypingAny = Any,
+    sender: TypingAny = Anonymous,
+    *arguments: TypingAny,
+    **named: TypingAny,
+) -> list[tuple[TypingAny, TypingAny]]:
     """Like pydispatcher.robust.sendRobust but it also logs errors and returns
     Failures instead of exceptions.
     """
     dont_log = named.pop("dont_log", ())
-    dont_log = (
-        tuple(dont_log)
-        if isinstance(dont_log, collections.abc.Sequence)
-        else (dont_log,)
-    )
+    dont_log = tuple(dont_log) if isinstance(dont_log, Sequence) else (dont_log,)
     dont_log += (StopDownload,)
     spider = named.get("spider", None)
-    responses = []
+    responses: list[tuple[TypingAny, TypingAny]] = []
     for receiver in liveReceivers(getAllReceivers(sender, signal)):
+        result: TypingAny
         try:
             response = robustApply(
                 receiver, signal=signal, sender=sender, *arguments, **named
@@ -60,13 +66,18 @@ def send_catch_log(signal=Any, sender=Anonymous, *arguments, **named):
     return responses
 
 
-def send_catch_log_deferred(signal=Any, sender=Anonymous, *arguments, **named):
+def send_catch_log_deferred(
+    signal: TypingAny = Any,
+    sender: TypingAny = Anonymous,
+    *arguments: TypingAny,
+    **named: TypingAny,
+) -> Deferred[list[tuple[TypingAny, TypingAny]]]:
     """Like send_catch_log but supports returning deferreds on signal handlers.
     Returns a deferred that gets fired once all signal handlers deferreds were
     fired.
     """
 
-    def logerror(failure, recv):
+    def logerror(failure: Failure, recv: Any) -> Failure:
         if dont_log is None or not isinstance(failure.value, dont_log):
             logger.error(
                 "Error caught on signal handler: %(receiver)s",
@@ -78,20 +89,28 @@ def send_catch_log_deferred(signal=Any, sender=Anonymous, *arguments, **named):
 
     dont_log = named.pop("dont_log", None)
     spider = named.get("spider", None)
-    dfds = []
+    dfds: list[Deferred[tuple[TypingAny, TypingAny]]] = []
     for receiver in liveReceivers(getAllReceivers(sender, signal)):
-        d = maybeDeferred_coro(
+        d: Deferred[TypingAny] = maybeDeferred_coro(
             robustApply, receiver, signal=signal, sender=sender, *arguments, **named
         )
         d.addErrback(logerror, receiver)
-        d.addBoth(lambda result: (receiver, result))
-        dfds.append(d)
-    d = DeferredList(dfds)
-    d.addCallback(lambda out: [x[1] for x in out])
-    return d
+        # TODO https://pylint.readthedocs.io/en/latest/user_guide/messages/warning/cell-var-from-loop.html
+        d2: Deferred[tuple[TypingAny, TypingAny]] = d.addBoth(
+            lambda result: (
+                receiver,  # pylint: disable=cell-var-from-loop  # noqa: B023
+                result,
+            )
+        )
+        dfds.append(d2)
+    dl = DeferredList(dfds)
+    d3: Deferred[list[tuple[TypingAny, TypingAny]]] = dl.addCallback(
+        lambda out: [x[1] for x in out]
+    )
+    return d3
 
 
-def disconnect_all(signal=Any, sender=Any):
+def disconnect_all(signal: TypingAny = Any, sender: TypingAny = Any) -> None:
     """Disconnect all signal handlers. Useful for cleaning up after running
     tests
     """
