@@ -1,4 +1,5 @@
 from twisted.internet import reactor  # noqa: F401
+from twisted.python import log
 
 import scrapy
 from scrapy.crawler import CrawlerProcess
@@ -17,5 +18,6 @@ process = CrawlerProcess(
     }
 )
 
-process.crawl(NoRequestsSpider)
+d = process.crawl(NoRequestsSpider)
+d.addErrback(log.err)
 process.start()
