@@ -1,5 +1,6 @@
 from twisted.internet.main import installReactor
 from twisted.internet.selectreactor import SelectReactor
+from twisted.python import log
 
 import scrapy
 from scrapy.crawler import CrawlerProcess
@@ -26,5 +27,6 @@ process = CrawlerProcess(
     }
 )
 
-process.crawl(NoRequestsSpider)
+d = process.crawl(NoRequestsSpider)
+d.addErrback(log.err)
 process.start()
