@@ -20,7 +20,7 @@ class CmdlineTest(unittest.TestCase):
         self.env["SCRAPY_SETTINGS_MODULE"] = "tests.test_cmdline.settings"
 
     def _execute(self, *new_args, **kwargs):
-        encoding = getattr(sys.stdout, "encoding") or "utf-8"
+        encoding = sys.stdout.encoding or "utf-8"
         args = (sys.executable, "-m", "scrapy.cmdline") + new_args
         proc = Popen(args, stdout=PIPE, stderr=PIPE, env=self.env, **kwargs)
         comm = proc.communicate()[0].strip()
