@@ -8,6 +8,7 @@ from scrapy import FormRequest
 from scrapy.contracts import Contract, ContractsManager
 from scrapy.contracts.default import (
     CallbackKeywordArgumentsContract,
+    MetadataContract,
     ReturnsContract,
     ScrapesContract,
     UrlContract,
@@ -29,6 +30,10 @@ class ResponseMock:
     url = "http://scrapy.org"
 
 
+class ResponseMetaMock(ResponseMock):
+    meta = None
+
+
 class CustomSuccessContract(Contract):
     name = "custom_success_contract"
 
@@ -182,6 +187,46 @@ class TestSpider(Spider):
         """
         pass
 
+    def invalid_regex(self, response):
+        """method with invalid regex
+        @ Scrapy is awsome
+        """
+        pass
+
+    def invalid_regex_with_valid_contract(self, response):
+        """method with invalid regex
+        @ scrapy is awsome
+        @url http://scrapy.org
+        """
+        pass
+
+    def returns_request_meta(self, response):
+        """method which returns request
+        @url https://example.org
+        @meta {"cookiejar": "session1"}
+        @returns requests 1
+        """
+        return Request(
+            "https://example.org", meta=response.meta, callback=self.returns_item_meta
+        )
+
+    def returns_item_meta(self, response):
+        """method which returns item
+        @url http://scrapy.org
+        @meta {"key": "example"}
+        @returns items 1 1
+        """
+        return TestItem(name="example", url=response.url)
+
+    def returns_error_missing_meta(self, response):
+        """method which depends of metadata be defined
+
+        @url http://scrapy.org
+        @returns items 1
+        """
+        key = response.meta["key"]
+        yield {key: "value"}
+
 
 class CustomContractSuccessSpider(Spider):
     name = "custom_contract_success_spider"
@@ -211,6 +256,7 @@ class ContractsManagerTest(unittest.TestCase):
     contracts = [
         UrlContract,
         CallbackKeywordArgumentsContract,
+        MetadataContract,
         ReturnsContract,
         ScrapesContract,
         CustomFormContract,
@@ -315,6 +361,52 @@ class ContractsManagerTest(unittest.TestCase):
         request.callback(response, **request.cb_kwargs)
         self.should_error()
 
+    def test_meta(self):
+        spider = TestSpider()
+
+        # extract contracts correctly
+        contracts = self.conman.extract_contracts(spider.returns_request_meta)
+        self.assertEqual(len(contracts), 3)
+        self.assertEqual(
+            frozenset(type(x) for x in contracts),
+            frozenset([UrlContract, MetadataContract, ReturnsContract]),
+        )
+
+        contracts = self.conman.extract_contracts(spider.returns_item_meta)
+        self.assertEqual(len(contracts), 3)
+        self.assertEqual(
+            frozenset(type(x) for x in contracts),
+            frozenset([UrlContract, MetadataContract, ReturnsContract]),
+        )
+
+        response = ResponseMetaMock()
+
+        # returns_request
+        request = self.conman.from_method(spider.returns_request_meta, self.results)
+        assert request.meta["cookiejar"] == "session1"
+        response.meta = request.meta
+        request.callback(response)
+        assert response.meta["cookiejar"] == "session1"
+        self.should_succeed()
+
+        response = ResponseMetaMock()
+
+        # returns_item
+        request = self.conman.from_method(spider.returns_item_meta, self.results)
+        assert request.meta["key"] == "example"
+        response.meta = request.meta
+        request.callback(ResponseMetaMock)
+        assert response.meta["key"] == "example"
+        self.should_succeed()
+
+        response = ResponseMetaMock()
+
+        request = self.conman.from_method(
+            spider.returns_error_missing_meta, self.results
+        )
+        request.callback(response)
+        self.should_error()
+
     def test_returns(self):
         spider = TestSpider()
         response = ResponseMock()
@@ -385,6 +477,21 @@ class ContractsManagerTest(unittest.TestCase):
         message = "ContractFail: Missing fields: name, url"
         assert message in self.results.failures[-1][-1]
 
+    def test_regex(self):
+        spider = TestSpider()
+        response = ResponseMock()
+
+        # invalid regex
+        request = self.conman.from_method(spider.invalid_regex, self.results)
+        self.should_succeed()
+
+        # invalid regex with valid contract
+        request = self.conman.from_method(
+            spider.invalid_regex_with_valid_contract, self.results
+        )
+        self.should_succeed()
+        request.callback(response)
+
     def test_custom_contracts(self):
         self.conman.from_spider(CustomContractSuccessSpider(), self.results)
         self.should_succeed()
@@ -449,3 +556,61 @@ class ContractsManagerTest(unittest.TestCase):
 
         requests = self.conman.from_spider(spider, self.results)
         self.assertTrue(requests)
+
+
+class CustomFailContractPreProcess(Contract):
+    name = "test_contract"
+
+    def pre_process(self, response):
+        raise KeyboardInterrupt("Pre-process exception")
+
+
+class CustomFailContractPostProcess(Contract):
+    name = "test_contract"
+
+    def post_process(self, response):
+        raise KeyboardInterrupt("Post-process exception")
+
+
+class CustomContractPrePostProcess(unittest.TestCase):
+
+    def setUp(self):
+        self.results = TextTestResult(stream=None, descriptions=False, verbosity=0)
+
+    def test_pre_hook_keyboard_interrupt(self):
+        spider = TestSpider()
+        response = ResponseMock()
+        contract = CustomFailContractPreProcess(spider.returns_request)
+        conman = ContractsManager([contract])
+
+        try:
+            request = conman.from_method(spider.returns_request, self.results)
+            contract.add_pre_hook(request, self.results)
+            # Expect this to raise a KeyboardInterrupt
+            request.callback(response, **request.cb_kwargs)
+        except KeyboardInterrupt as e:
+            self.assertEqual(str(e), "Pre-process exception")
+        else:
+            self.fail("KeyboardInterrupt not raised")
+
+        self.assertFalse(self.results.failures)
+        self.assertFalse(self.results.errors)
+
+    def test_post_hook_keyboard_interrupt(self):
+        spider = TestSpider()
+        response = ResponseMock()
+        contract = CustomFailContractPostProcess(spider.returns_request)
+        conman = ContractsManager([contract])
+
+        try:
+            request = conman.from_method(spider.returns_request, self.results)
+            contract.add_post_hook(request, self.results)
+            # Expect this to raise a KeyboardInterrupt
+            request.callback(response, **request.cb_kwargs)
+        except KeyboardInterrupt as e:
+            self.assertEqual(str(e), "Post-process exception")
+        else:
+            self.fail("KeyboardInterrupt not raised")
+
+        self.assertFalse(self.results.failures)
+        self.assertFalse(self.results.errors)
