@@ -7,5 +7,6 @@ class SlotTest(unittest.TestCase):
     def test_repr(self):
         slot = Slot(concurrency=8, delay=0.1, randomize_delay=True)
         self.assertEqual(
-            repr(slot), "Slot(concurrency=8, delay=0.10, randomize_delay=True)"
+            repr(slot),
+            "Slot(concurrency=8, delay=0.10, randomize_delay=True)",
         )
