@@ -1,12 +1,14 @@
+from __future__ import annotations
+
 import contextlib
 import os
 import shutil
 import sys
-import tempfile
 from pathlib import Path
-from typing import Optional, Type
+from tempfile import mkdtemp, mkstemp
 from unittest import SkipTest, mock
 
+import pytest
 from testfixtures import LogCapture
 from twisted.cred import checkers, credentials, portal
 from twisted.internet import defer, error, reactor
@@ -29,9 +31,9 @@ from scrapy.http import Headers, HtmlResponse, Request
 from scrapy.http.response.text import TextResponse
 from scrapy.responsetypes import responsetypes
 from scrapy.spiders import Spider
-from scrapy.utils.misc import create_instance
+from scrapy.utils.misc import build_from_crawler
 from scrapy.utils.python import to_bytes
-from scrapy.utils.test import get_crawler, skip_if_no_boto
+from scrapy.utils.test import get_crawler
 from tests import NON_EXISTING_RESOLVABLE
 from tests.mockserver import (
     Echo,
@@ -107,13 +109,14 @@ class LoadTestCase(unittest.TestCase):
 class FileTestCase(unittest.TestCase):
     def setUp(self):
         # add a special char to check that they are handled correctly
-        self.tmpname = Path(self.mktemp() + "^")
+        self.fd, self.tmpname = mkstemp(suffix="^")
         Path(self.tmpname).write_text("0123456789", encoding="utf-8")
-        handler = create_instance(FileDownloadHandler, None, get_crawler())
+        handler = build_from_crawler(FileDownloadHandler, get_crawler())
         self.download_request = handler.download_request
 
     def tearDown(self):
-        self.tmpname.unlink()
+        os.close(self.fd)
+        os.remove(self.tmpname)
 
     def test_download(self):
         def _test(response):
@@ -122,12 +125,12 @@ class FileTestCase(unittest.TestCase):
             self.assertEqual(response.body, b"0123456789")
             self.assertEqual(response.protocol, None)
 
-        request = Request(path_to_file_uri(str(self.tmpname)))
+        request = Request(path_to_file_uri(self.tmpname))
         assert request.url.upper().endswith("%5E")
         return self.download_request(request, Spider("foo")).addCallback(_test)
 
     def test_non_existent(self):
-        request = Request(path_to_file_uri(self.mktemp()))
+        request = Request(path_to_file_uri(mkdtemp()))
         d = self.download_request(request, Spider("foo"))
         return self.assertFailure(d, OSError)
 
@@ -217,15 +220,14 @@ class DuplicateHeaderResource(resource.Resource):
 
 class HttpTestCase(unittest.TestCase):
     scheme = "http"
-    download_handler_cls: Type = HTTPDownloadHandler
+    download_handler_cls: type = HTTPDownloadHandler
 
     # only used for HTTPS tests
     keyfile = "keys/localhost.key"
     certfile = "keys/localhost.crt"
 
     def setUp(self):
-        self.tmpname = Path(self.mktemp())
-        self.tmpname.mkdir()
+        self.tmpname = Path(mkdtemp())
         (self.tmpname / "file").write_bytes(b"0123456789")
         r = static.File(str(self.tmpname))
         r.putChild(b"redirect", util.Redirect(b"/file"))
@@ -257,8 +259,8 @@ class HttpTestCase(unittest.TestCase):
         else:
             self.port = reactor.listenTCP(0, self.wrapper, interface=self.host)
         self.portno = self.port.getHost().port
-        self.download_handler = create_instance(
-            self.download_handler_cls, None, get_crawler()
+        self.download_handler = build_from_crawler(
+            self.download_handler_cls, get_crawler()
         )
         self.download_request = self.download_handler.download_request
 
@@ -428,7 +430,7 @@ class HttpTestCase(unittest.TestCase):
 class Http10TestCase(HttpTestCase):
     """HTTP 1.0 test case"""
 
-    download_handler_cls: Type = HTTP10DownloadHandler
+    download_handler_cls: type = HTTP10DownloadHandler
 
     def test_protocol(self):
         request = Request(self.getURL("host"), method="GET")
@@ -445,7 +447,7 @@ class Https10TestCase(Http10TestCase):
 class Http11TestCase(HttpTestCase):
     """HTTP 1.1 test case"""
 
-    download_handler_cls: Type = HTTP11DownloadHandler
+    download_handler_cls: type = HTTP11DownloadHandler
 
     def test_download_without_maxsize_limit(self):
         request = Request(self.getURL("file"))
@@ -557,7 +559,7 @@ class Http11TestCase(HttpTestCase):
 
     def test_download_broken_content_allow_data_loss_via_setting(self, url="broken"):
         crawler = get_crawler(settings_dict={"DOWNLOAD_FAIL_ON_DATALOSS": False})
-        download_handler = create_instance(self.download_handler_cls, None, crawler)
+        download_handler = build_from_crawler(self.download_handler_cls, crawler)
         request = Request(self.getURL(url))
         d = download_handler.download_request(request, Spider("foo"))
         d.addCallback(lambda r: r.flags)
@@ -590,7 +592,7 @@ class Https11TestCase(Http11TestCase):
         crawler = get_crawler(
             settings_dict={"DOWNLOADER_CLIENT_TLS_VERBOSE_LOGGING": True}
         )
-        download_handler = create_instance(self.download_handler_cls, None, crawler)
+        download_handler = build_from_crawler(self.download_handler_cls, crawler)
         try:
             with LogCapture() as log_capture:
                 request = Request(self.getURL("file"))
@@ -645,14 +647,13 @@ class Https11InvalidDNSPattern(Https11TestCase):
 
 class Https11CustomCiphers(unittest.TestCase):
     scheme = "https"
-    download_handler_cls: Type = HTTP11DownloadHandler
+    download_handler_cls: type = HTTP11DownloadHandler
 
     keyfile = "keys/localhost.key"
     certfile = "keys/localhost.crt"
 
     def setUp(self):
-        self.tmpname = Path(self.mktemp())
-        self.tmpname.mkdir()
+        self.tmpname = Path(mkdtemp())
         (self.tmpname / "file").write_bytes(b"0123456789")
         r = static.File(str(self.tmpname))
         self.site = server.Site(r, timeout=None)
@@ -669,9 +670,7 @@ class Https11CustomCiphers(unittest.TestCase):
         crawler = get_crawler(
             settings_dict={"DOWNLOADER_CLIENT_TLS_CIPHERS": "CAMELLIA256-SHA"}
         )
-        self.download_handler = create_instance(
-            self.download_handler_cls, None, crawler
-        )
+        self.download_handler = build_from_crawler(self.download_handler_cls, crawler)
         self.download_request = self.download_handler.download_request
 
     @defer.inlineCallbacks
@@ -695,7 +694,7 @@ class Https11CustomCiphers(unittest.TestCase):
 class Http11MockServerTestCase(unittest.TestCase):
     """HTTP 1.1 test case with MockServer"""
 
-    settings_dict: Optional[dict] = None
+    settings_dict: dict | None = None
 
     def setUp(self):
         self.mockserver = MockServer()
@@ -726,33 +725,6 @@ class Http11MockServerTestCase(unittest.TestCase):
         reason = crawler.spider.meta["close_reason"]
         self.assertTrue(reason, "finished")
 
-    @defer.inlineCallbacks
-    def test_download_gzip_response(self):
-        crawler = get_crawler(SingleRequestSpider, self.settings_dict)
-        body = b"1" * 100  # PayloadResource requires body length to be 100
-        request = Request(
-            self.mockserver.url("/payload"),
-            method="POST",
-            body=body,
-            meta={"download_maxsize": 50},
-        )
-        yield crawler.crawl(seed=request)
-        failure = crawler.spider.meta["failure"]
-        # download_maxsize < 100, hence the CancelledError
-        self.assertIsInstance(failure.value, defer.CancelledError)
-
-        # See issue https://twistedmatrix.com/trac/ticket/8175
-        raise unittest.SkipTest("xpayload fails on PY3")
-        crawler = get_crawler(SingleRequestSpider, self.settings_dict)
-        request.headers.setdefault(b"Accept-Encoding", b"gzip,deflate")
-        request = request.replace(url=self.mockserver.url("/xpayload"))
-        yield crawler.crawl(seed=request)
-        # download_maxsize = 50 is enough for the gzipped response
-        failure = crawler.spider.meta.get("failure")
-        self.assertIsNone(failure)
-        reason = crawler.spider.meta["close_reason"]
-        self.assertTrue(reason, "finished")
-
 
 class UriResource(resource.Resource):
     """Return the full uri that was requested"""
@@ -770,7 +742,7 @@ class UriResource(resource.Resource):
 
 
 class HttpProxyTestCase(unittest.TestCase):
-    download_handler_cls: Type = HTTPDownloadHandler
+    download_handler_cls: type = HTTPDownloadHandler
     expected_http_proxy_request_body = b"http://example.com"
 
     def setUp(self):
@@ -778,8 +750,8 @@ class HttpProxyTestCase(unittest.TestCase):
         wrapper = WrappingFactory(site)
         self.port = reactor.listenTCP(0, wrapper, interface="127.0.0.1")
         self.portno = self.port.getHost().port
-        self.download_handler = create_instance(
-            self.download_handler_cls, None, get_crawler()
+        self.download_handler = build_from_crawler(
+            self.download_handler_cls, get_crawler()
         )
         self.download_request = self.download_handler.download_request
 
@@ -813,14 +785,14 @@ class HttpProxyTestCase(unittest.TestCase):
 
 
 class Http10ProxyTestCase(HttpProxyTestCase):
-    download_handler_cls: Type = HTTP10DownloadHandler
+    download_handler_cls: type = HTTP10DownloadHandler
 
     def test_download_with_proxy_https_noconnect(self):
         raise unittest.SkipTest("noconnect is not supported in HTTP10DownloadHandler")
 
 
 class Http11ProxyTestCase(HttpProxyTestCase):
-    download_handler_cls: Type = HTTP11DownloadHandler
+    download_handler_cls: type = HTTP11DownloadHandler
 
     @defer.inlineCallbacks
     def test_download_with_proxy_https_timeout(self):
@@ -853,14 +825,13 @@ class HttpDownloadHandlerMock:
         return request
 
 
+@pytest.mark.requires_botocore
 class S3AnonTestCase(unittest.TestCase):
     def setUp(self):
-        skip_if_no_boto()
         crawler = get_crawler()
-        self.s3reqh = create_instance(
-            objcls=S3DownloadHandler,
-            settings=None,
-            crawler=crawler,
+        self.s3reqh = build_from_crawler(
+            S3DownloadHandler,
+            crawler,
             httpdownloadhandler=HttpDownloadHandlerMock,
             # anon=True, # implicit
         )
@@ -875,8 +846,9 @@ class S3AnonTestCase(unittest.TestCase):
         self.assertEqual(httpreq.url, "http://aws-publicdatasets.s3.amazonaws.com/")
 
 
+@pytest.mark.requires_botocore
 class S3TestCase(unittest.TestCase):
-    download_handler_cls: Type = S3DownloadHandler
+    download_handler_cls: type = S3DownloadHandler
 
     # test use same example keys than amazon developer guide
     # http://s3.amazonaws.com/awsdocs/S3/20060301/s3-dg-20060301.pdf
@@ -886,12 +858,10 @@ class S3TestCase(unittest.TestCase):
     AWS_SECRET_ACCESS_KEY = "uV3F3YluFJax1cknvbcGwgjvx4QpvB+leU8dUj2o"
 
     def setUp(self):
-        skip_if_no_boto()
         crawler = get_crawler()
-        s3reqh = create_instance(
-            objcls=S3DownloadHandler,
-            settings=None,
-            crawler=crawler,
+        s3reqh = build_from_crawler(
+            S3DownloadHandler,
+            crawler,
             aws_access_key_id=self.AWS_ACCESS_KEY_ID,
             aws_secret_access_key=self.AWS_SECRET_ACCESS_KEY,
             httpdownloadhandler=HttpDownloadHandlerMock,
@@ -916,16 +886,15 @@ class S3TestCase(unittest.TestCase):
     def test_extra_kw(self):
         try:
             crawler = get_crawler()
-            create_instance(
-                objcls=S3DownloadHandler,
-                settings=None,
-                crawler=crawler,
+            build_from_crawler(
+                S3DownloadHandler,
+                crawler,
                 extra_kw=True,
             )
         except Exception as e:
             self.assertIsInstance(e, (TypeError, NotConfigured))
         else:
-            assert False
+            raise AssertionError()
 
     def test_request_signing1(self):
         # gets an object from the johnsmith bucket.
@@ -1047,8 +1016,7 @@ class BaseFTPTestCase(unittest.TestCase):
         from scrapy.core.downloader.handlers.ftp import FTPDownloadHandler
 
         # setup dirs and test file
-        self.directory = Path(self.mktemp())
-        self.directory.mkdir()
+        self.directory = Path(mkdtemp())
         userdir = self.directory / self.username
         userdir.mkdir()
         for filename, content in self.test_files:
@@ -1066,9 +1034,7 @@ class BaseFTPTestCase(unittest.TestCase):
         self.port = reactor.listenTCP(0, self.factory, interface="127.0.0.1")
         self.portNum = self.port.getHost().port
         crawler = get_crawler()
-        self.download_handler = create_instance(
-            FTPDownloadHandler, crawler.settings, crawler
-        )
+        self.download_handler = build_from_crawler(FTPDownloadHandler, crawler)
         self.addCleanup(self.port.stopListening)
 
     def tearDown(self):
@@ -1126,7 +1092,7 @@ class BaseFTPTestCase(unittest.TestCase):
         return self._add_test_callbacks(d, _test)
 
     def test_ftp_local_filename(self):
-        f, local_fname = tempfile.mkstemp()
+        f, local_fname = mkstemp()
         fname_bytes = to_bytes(local_fname)
         local_fname = Path(local_fname)
         os.close(f)
@@ -1147,7 +1113,7 @@ class BaseFTPTestCase(unittest.TestCase):
         return self._add_test_callbacks(d, _test)
 
     def _test_response_class(self, filename, response_class):
-        f, local_fname = tempfile.mkstemp()
+        f, local_fname = mkstemp()
         local_fname = Path(local_fname)
         os.close(f)
         meta = {}
@@ -1197,9 +1163,7 @@ class AnonymousFTPTestCase(BaseFTPTestCase):
         from scrapy.core.downloader.handlers.ftp import FTPDownloadHandler
 
         # setup dir and test file
-        self.directory = Path(self.mktemp())
-        self.directory.mkdir()
-
+        self.directory = Path(mkdtemp())
         for filename, content in self.test_files:
             (self.directory / filename).write_bytes(content)
 
@@ -1212,9 +1176,7 @@ class AnonymousFTPTestCase(BaseFTPTestCase):
         self.port = reactor.listenTCP(0, self.factory, interface="127.0.0.1")
         self.portNum = self.port.getHost().port
         crawler = get_crawler()
-        self.download_handler = create_instance(
-            FTPDownloadHandler, crawler.settings, crawler
-        )
+        self.download_handler = build_from_crawler(FTPDownloadHandler, crawler)
         self.addCleanup(self.port.stopListening)
 
     def tearDown(self):
@@ -1224,9 +1186,7 @@ class AnonymousFTPTestCase(BaseFTPTestCase):
 class DataURITestCase(unittest.TestCase):
     def setUp(self):
         crawler = get_crawler()
-        self.download_handler = create_instance(
-            DataURIDownloadHandler, crawler.settings, crawler
-        )
+        self.download_handler = build_from_crawler(DataURIDownloadHandler, crawler)
         self.download_request = self.download_handler.download_request
         self.spider = Spider("foo")
 
