@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import json
 import random
 import re
@@ -5,7 +7,8 @@ import shutil
 import string
 from ipaddress import IPv4Address
 from pathlib import Path
-from typing import Dict
+from tempfile import mkdtemp
+from typing import TYPE_CHECKING
 from unittest import mock, skipIf
 from urllib.parse import urlencode
 
@@ -19,7 +22,6 @@ from twisted.internet.defer import (
 from twisted.internet.endpoints import SSL4ClientEndpoint, SSL4ServerEndpoint
 from twisted.internet.error import TimeoutError
 from twisted.internet.ssl import Certificate, PrivateCertificate, optionsForClientTLS
-from twisted.python.failure import Failure
 from twisted.trial.unittest import TestCase
 from twisted.web.client import URI, ResponseFailed
 from twisted.web.http import H2_ENABLED
@@ -32,6 +34,9 @@ from scrapy.settings import Settings
 from scrapy.spiders import Spider
 from tests.mockserver import LeafResource, Status, ssl_context_factory
 
+if TYPE_CHECKING:
+    from twisted.python.failure import Failure
+
 
 def generate_random_string(size):
     return "".join(random.choices(string.ascii_uppercase + string.digits, k=size))
@@ -147,7 +152,7 @@ class QueryParams(LeafResource):
         request.setHeader("Content-Type", "application/json; charset=UTF-8")
         request.setHeader("Content-Encoding", "UTF-8")
 
-        query_params: Dict[str, str] = {}
+        query_params: dict[str, str] = {}
         assert request.args is not None
         for k, v in request.args.items():
             query_params[str(k, "utf-8")] = str(v[0], "utf-8")
@@ -185,8 +190,7 @@ class Https2ClientProtocolTestCase(TestCase):
     certificate_file = Path(__file__).parent / "keys" / "localhost.crt"
 
     def _init_resource(self):
-        self.temp_directory = self.mktemp()
-        Path(self.temp_directory).mkdir()
+        self.temp_directory = mkdtemp()
         r = File(self.temp_directory)
         r.putChild(b"get-data-html-small", GetDataHtmlSmall())
         r.putChild(b"get-data-html-large", GetDataHtmlLarge())
@@ -275,7 +279,9 @@ class Https2ClientProtocolTestCase(TestCase):
             self.assertEqual(response.body, expected_body)
             self.assertEqual(response.request, request)
 
-            content_length = int(response.headers.get("Content-Length"))
+            content_length_header = response.headers.get("Content-Length")
+            assert content_length_header is not None
+            content_length = int(content_length_header)
             self.assertEqual(len(response.body), content_length)
 
         d = self.make_request(request)
@@ -320,11 +326,15 @@ class Https2ClientProtocolTestCase(TestCase):
             self.assertEqual(response.status, expected_status)
             self.assertEqual(response.request, request)
 
-            content_length = int(response.headers.get("Content-Length"))
+            content_length_header = response.headers.get("Content-Length")
+            assert content_length_header is not None
+            content_length = int(content_length_header)
             self.assertEqual(len(response.body), content_length)
 
             # Parse the body
-            content_encoding = str(response.headers[b"Content-Encoding"], "utf-8")
+            content_encoding_header = response.headers[b"Content-Encoding"]
+            assert content_encoding_header is not None
+            content_encoding = str(content_encoding_header, "utf-8")
             body = json.loads(str(response.body, content_encoding))
             self.assertIn("request-body", body)
             self.assertIn("extra-data", body)
@@ -562,7 +572,9 @@ class Https2ClientProtocolTestCase(TestCase):
         request = Request(self.get_url(f"/query-params?{urlencode(params)}"))
 
         def assert_query_params(response: Response):
-            content_encoding = str(response.headers[b"Content-Encoding"], "utf-8")
+            content_encoding_header = response.headers[b"Content-Encoding"]
+            assert content_encoding_header is not None
+            content_encoding = str(content_encoding_header, "utf-8")
             data = json.loads(str(response.body, content_encoding))
             self.assertEqual(data, params)
 
@@ -592,6 +604,7 @@ class Https2ClientProtocolTestCase(TestCase):
         def assert_metadata(response: Response):
             self.assertEqual(response.request, request)
             self.assertIsInstance(response.certificate, Certificate)
+            assert response.certificate  # typing
             self.assertIsNotNone(response.certificate.original)
             self.assertEqual(
                 response.certificate.getIssuer(), self.client_certificate.getIssuer()
