@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import pickle
 import re
 import unittest
@@ -15,7 +17,7 @@ from tests import get_testdata
 # a hack to skip base class tests in pytest
 class Base:
     class LinkExtractorTestCase(unittest.TestCase):
-        extractor_cls = None
+        extractor_cls: type | None = None
 
         def setUp(self):
             body = get_testdata("link_extractor", "linkextractor.html")
@@ -36,7 +38,7 @@ class Base:
             page4_url = "http://example.com/page%204.html"
 
             self.assertEqual(
-                [link for link in lx.extract_links(self.response)],
+                list(lx.extract_links(self.response)),
                 [
                     Link(url="http://example.com/sample1.html", text=""),
                     Link(url="http://example.com/sample2.html", text="sample 2"),
@@ -54,7 +56,7 @@ class Base:
         def test_extract_filter_allow(self):
             lx = self.extractor_cls(allow=("sample",))
             self.assertEqual(
-                [link for link in lx.extract_links(self.response)],
+                list(lx.extract_links(self.response)),
                 [
                     Link(url="http://example.com/sample1.html", text=""),
                     Link(url="http://example.com/sample2.html", text="sample 2"),
@@ -69,7 +71,7 @@ class Base:
         def test_extract_filter_allow_with_duplicates(self):
             lx = self.extractor_cls(allow=("sample",), unique=False)
             self.assertEqual(
-                [link for link in lx.extract_links(self.response)],
+                list(lx.extract_links(self.response)),
                 [
                     Link(url="http://example.com/sample1.html", text=""),
                     Link(url="http://example.com/sample2.html", text="sample 2"),
@@ -92,7 +94,7 @@ class Base:
         def test_extract_filter_allow_with_duplicates_canonicalize(self):
             lx = self.extractor_cls(allow=("sample",), unique=False, canonicalize=True)
             self.assertEqual(
-                [link for link in lx.extract_links(self.response)],
+                list(lx.extract_links(self.response)),
                 [
                     Link(url="http://example.com/sample1.html", text=""),
                     Link(url="http://example.com/sample2.html", text="sample 2"),
@@ -115,7 +117,7 @@ class Base:
         def test_extract_filter_allow_no_duplicates_canonicalize(self):
             lx = self.extractor_cls(allow=("sample",), unique=True, canonicalize=True)
             self.assertEqual(
-                [link for link in lx.extract_links(self.response)],
+                list(lx.extract_links(self.response)),
                 [
                     Link(url="http://example.com/sample1.html", text=""),
                     Link(url="http://example.com/sample2.html", text="sample 2"),
@@ -126,7 +128,7 @@ class Base:
         def test_extract_filter_allow_and_deny(self):
             lx = self.extractor_cls(allow=("sample",), deny=("3",))
             self.assertEqual(
-                [link for link in lx.extract_links(self.response)],
+                list(lx.extract_links(self.response)),
                 [
                     Link(url="http://example.com/sample1.html", text=""),
                     Link(url="http://example.com/sample2.html", text="sample 2"),
@@ -136,7 +138,7 @@ class Base:
         def test_extract_filter_allowed_domains(self):
             lx = self.extractor_cls(allow_domains=("google.com",))
             self.assertEqual(
-                [link for link in lx.extract_links(self.response)],
+                list(lx.extract_links(self.response)),
                 [
                     Link(url="http://www.google.com/something", text=""),
                 ],
@@ -147,7 +149,7 @@ class Base:
 
             lx = self.extractor_cls(allow="sample")
             self.assertEqual(
-                [link for link in lx.extract_links(self.response)],
+                list(lx.extract_links(self.response)),
                 [
                     Link(url="http://example.com/sample1.html", text=""),
                     Link(url="http://example.com/sample2.html", text="sample 2"),
@@ -161,7 +163,7 @@ class Base:
 
             lx = self.extractor_cls(allow="sample", deny="3")
             self.assertEqual(
-                [link for link in lx.extract_links(self.response)],
+                list(lx.extract_links(self.response)),
                 [
                     Link(url="http://example.com/sample1.html", text=""),
                     Link(url="http://example.com/sample2.html", text="sample 2"),
@@ -170,7 +172,7 @@ class Base:
 
             lx = self.extractor_cls(allow_domains="google.com")
             self.assertEqual(
-                [link for link in lx.extract_links(self.response)],
+                list(lx.extract_links(self.response)),
                 [
                     Link(url="http://www.google.com/something", text=""),
                 ],
@@ -178,14 +180,14 @@ class Base:
 
             lx = self.extractor_cls(deny_domains="example.com")
             self.assertEqual(
-                [link for link in lx.extract_links(self.response)],
+                list(lx.extract_links(self.response)),
                 [
                     Link(url="http://www.google.com/something", text=""),
                 ],
             )
 
         def test_nofollow(self):
-            '''Test the extractor's behaviour for links with rel="nofollow"'''
+            """Test the extractor's behaviour for links with rel='nofollow'"""
 
             html = b"""<html><head><title>Page title<title>
             <body>
@@ -235,20 +237,20 @@ class Base:
             url2 = "http://evenmorestuff.com/uglystuff/index"
 
             lx = self.extractor_cls(allow=(r"stuff1",))
-            self.assertEqual(lx.matches(url1), True)
-            self.assertEqual(lx.matches(url2), False)
+            self.assertTrue(lx.matches(url1))
+            self.assertFalse(lx.matches(url2))
 
             lx = self.extractor_cls(deny=(r"uglystuff",))
-            self.assertEqual(lx.matches(url1), True)
-            self.assertEqual(lx.matches(url2), False)
+            self.assertTrue(lx.matches(url1))
+            self.assertFalse(lx.matches(url2))
 
             lx = self.extractor_cls(allow_domains=("evenmorestuff.com",))
-            self.assertEqual(lx.matches(url1), False)
-            self.assertEqual(lx.matches(url2), True)
+            self.assertFalse(lx.matches(url1))
+            self.assertTrue(lx.matches(url2))
 
             lx = self.extractor_cls(deny_domains=("lotsofstuff.com",))
-            self.assertEqual(lx.matches(url1), False)
-            self.assertEqual(lx.matches(url2), True)
+            self.assertFalse(lx.matches(url1))
+            self.assertTrue(lx.matches(url2))
 
             lx = self.extractor_cls(
                 allow=["blah1"],
@@ -256,15 +258,15 @@ class Base:
                 allow_domains=["blah1.com"],
                 deny_domains=["blah2.com"],
             )
-            self.assertEqual(lx.matches("http://blah1.com/blah1"), True)
-            self.assertEqual(lx.matches("http://blah1.com/blah2"), False)
-            self.assertEqual(lx.matches("http://blah2.com/blah1"), False)
-            self.assertEqual(lx.matches("http://blah2.com/blah2"), False)
+            self.assertTrue(lx.matches("http://blah1.com/blah1"))
+            self.assertFalse(lx.matches("http://blah1.com/blah2"))
+            self.assertFalse(lx.matches("http://blah2.com/blah1"))
+            self.assertFalse(lx.matches("http://blah2.com/blah2"))
 
         def test_restrict_xpaths(self):
             lx = self.extractor_cls(restrict_xpaths=('//div[@id="subwrapper"]',))
             self.assertEqual(
-                [link for link in lx.extract_links(self.response)],
+                list(lx.extract_links(self.response)),
                 [
                     Link(url="http://example.com/sample1.html", text=""),
                     Link(url="http://example.com/sample2.html", text="sample 2"),
@@ -336,7 +338,7 @@ class Base:
                 restrict_css=("#subwrapper + a",),
             )
             self.assertEqual(
-                [link for link in lx.extract_links(self.response)],
+                list(lx.extract_links(self.response)),
                 [
                     Link(url="http://example.com/sample1.html", text=""),
                     Link(url="http://example.com/sample2.html", text="sample 2"),
@@ -434,8 +436,7 @@ class Base:
 
             def process_value(value):
                 m = re.search(r"javascript:goToPage\('(.*?)'", value)
-                if m:
-                    return m.group(1)
+                return m.group(1) if m else None
 
             lx = self.extractor_cls(process_value=process_value)
             self.assertEqual(
@@ -704,7 +705,7 @@ class Base:
             response = HtmlResponse("http://example.org/index.html", body=html)
             lx = self.extractor_cls()
             self.assertEqual(
-                [link for link in lx.extract_links(response)],
+                list(lx.extract_links(response)),
                 [
                     Link(
                         url="http://example.org/item1.html",
@@ -869,7 +870,7 @@ class LxmlLinkExtractorTestCase(Base.LinkExtractorTestCase):
         response = HtmlResponse("http://example.org/index.html", body=html)
         lx = self.extractor_cls()
         self.assertEqual(
-            [link for link in lx.extract_links(response)],
+            list(lx.extract_links(response)),
             [
                 Link(
                     url="http://example.org/item1.html", text="Item 1", nofollow=False
@@ -890,7 +891,7 @@ class LxmlLinkExtractorTestCase(Base.LinkExtractorTestCase):
         # Simple text inclusion test
         lx = self.extractor_cls(restrict_text="dog")
         self.assertEqual(
-            [link for link in lx.extract_links(response)],
+            list(lx.extract_links(response)),
             [
                 Link(
                     url="http://example.org/item2.html",
@@ -902,7 +903,7 @@ class LxmlLinkExtractorTestCase(Base.LinkExtractorTestCase):
         # Unique regex test
         lx = self.extractor_cls(restrict_text=r"of.*dog")
         self.assertEqual(
-            [link for link in lx.extract_links(response)],
+            list(lx.extract_links(response)),
             [
                 Link(
                     url="http://example.org/item2.html",
@@ -914,7 +915,7 @@ class LxmlLinkExtractorTestCase(Base.LinkExtractorTestCase):
         # Multiple regex test
         lx = self.extractor_cls(restrict_text=[r"of.*dog", r"of.*cat"])
         self.assertEqual(
-            [link for link in lx.extract_links(response)],
+            list(lx.extract_links(response)),
             [
                 Link(
                     url="http://example.org/item1.html",
@@ -929,9 +930,6 @@ class LxmlLinkExtractorTestCase(Base.LinkExtractorTestCase):
             ],
         )
 
-    def test_restrict_xpaths_with_html_entities(self):
-        super().test_restrict_xpaths_with_html_entities()
-
     @mark.skipif(
         Version(w3lib_version) < Version("2.0.0"),
         reason=(
@@ -948,7 +946,7 @@ class LxmlLinkExtractorTestCase(Base.LinkExtractorTestCase):
         response = HtmlResponse("http://example.org/index.html", body=html)
         lx = self.extractor_cls()
         self.assertEqual(
-            [link for link in lx.extract_links(response)],
+            list(lx.extract_links(response)),
             [
                 Link(
                     url="http://example.org/item2.html",
@@ -962,3 +960,15 @@ class LxmlLinkExtractorTestCase(Base.LinkExtractorTestCase):
                 ),
             ],
         )
+
+    def test_link_allowed_is_false_with_empty_url(self):
+        bad_link = Link("")
+        self.assertFalse(LxmlLinkExtractor()._link_allowed(bad_link))
+
+    def test_link_allowed_is_false_with_bad_url_prefix(self):
+        bad_link = Link("htp://should_be_http.example")
+        self.assertFalse(LxmlLinkExtractor()._link_allowed(bad_link))
+
+    def test_link_allowed_is_false_with_missing_url_prefix(self):
+        bad_link = Link("should_have_prefix.example")
+        self.assertFalse(LxmlLinkExtractor()._link_allowed(bad_link))
