@@ -2,6 +2,7 @@ import dataclasses
 import os
 import random
 import time
+import warnings
 from datetime import datetime
 from io import BytesIO
 from pathlib import Path
@@ -11,6 +12,7 @@ from unittest import mock
 from urllib.parse import urlparse
 
 import attr
+import pytest
 from itemadapter import ItemAdapter
 from twisted.internet import defer
 from twisted.trial import unittest
@@ -24,13 +26,11 @@ from scrapy.pipelines.files import (
     GCSFilesStore,
     S3FilesStore,
 )
-from scrapy.settings import Settings
 from scrapy.utils.test import (
     assert_gcs_environ,
     get_crawler,
     get_ftp_content_and_delete,
     get_gcs_content_and_delete,
-    skip_if_no_boto,
 )
 from tests.mockserver import MockFTPServer
 
@@ -217,10 +217,10 @@ class FilesPipelineTestCase(unittest.TestCase):
             def file_path(self, request, response=None, info=None, item=None):
                 return f'full/{item.get("path")}'
 
-        file_path = CustomFilesPipeline.from_settings(
-            Settings({"FILES_STORE": self.tempdir})
+        file_path = CustomFilesPipeline.from_crawler(
+            get_crawler(None, {"FILES_STORE": self.tempdir})
         ).file_path
-        item = dict(path="path-to-store-file")
+        item = {"path": "path-to-store-file"}
         request = Request("http://example.com")
         self.assertEqual(file_path(request, item=item), "full/path-to-store-file")
 
@@ -235,7 +235,9 @@ class FilesPipelineTestCaseFieldsMixin:
     def test_item_fields_default(self):
         url = "http://www.example.com/files/1.txt"
         item = self.item_class(name="item1", file_urls=[url])
-        pipeline = FilesPipeline.from_settings(Settings({"FILES_STORE": self.tempdir}))
+        pipeline = FilesPipeline.from_crawler(
+            get_crawler(None, {"FILES_STORE": self.tempdir})
+        )
         requests = list(pipeline.get_media_requests(item, None))
         self.assertEqual(requests[0].url, url)
         results = [(True, {"url": url})]
@@ -247,13 +249,14 @@ class FilesPipelineTestCaseFieldsMixin:
     def test_item_fields_override_settings(self):
         url = "http://www.example.com/files/1.txt"
         item = self.item_class(name="item1", custom_file_urls=[url])
-        pipeline = FilesPipeline.from_settings(
-            Settings(
+        pipeline = FilesPipeline.from_crawler(
+            get_crawler(
+                None,
                 {
                     "FILES_STORE": self.tempdir,
                     "FILES_URLS_FIELD": "custom_file_urls",
                     "FILES_RESULT_FIELD": "custom_files",
-                }
+                },
             )
         )
         requests = list(pipeline.get_media_requests(item, None))
@@ -308,11 +311,11 @@ class FilesPipelineTestCaseFieldsDataClass(
 class FilesPipelineTestAttrsItem:
     name = attr.ib(default="")
     # default fields
-    file_urls = attr.ib(default=lambda: [])
-    files = attr.ib(default=lambda: [])
+    file_urls: list[str] = attr.ib(default=lambda: [])
+    files: list[dict[str, str]] = attr.ib(default=lambda: [])
     # overridden fields
-    custom_file_urls = attr.ib(default=lambda: [])
-    custom_files = attr.ib(default=lambda: [])
+    custom_file_urls: list[str] = attr.ib(default=lambda: [])
+    custom_files: list[dict[str, str]] = attr.ib(default=lambda: [])
 
 
 class FilesPipelineTestCaseFieldsAttrsItem(
@@ -371,8 +374,10 @@ class FilesPipelineTestCaseCustomSettings(unittest.TestCase):
         different settings.
         """
         custom_settings = self._generate_fake_settings()
-        another_pipeline = FilesPipeline.from_settings(Settings(custom_settings))
-        one_pipeline = FilesPipeline(self.tempdir)
+        another_pipeline = FilesPipeline.from_crawler(
+            get_crawler(None, custom_settings)
+        )
+        one_pipeline = FilesPipeline(self.tempdir, crawler=get_crawler(None))
         for pipe_attr, settings_attr, pipe_ins_attr in self.file_cls_attr_settings_map:
             default_value = self.default_cls_settings[pipe_attr]
             self.assertEqual(getattr(one_pipeline, pipe_attr), default_value)
@@ -385,7 +390,7 @@ class FilesPipelineTestCaseCustomSettings(unittest.TestCase):
         If subclasses override class attributes and there are no special settings those values should be kept.
         """
         pipe_cls = self._generate_fake_pipeline()
-        pipe = pipe_cls.from_settings(Settings({"FILES_STORE": self.tempdir}))
+        pipe = pipe_cls.from_crawler(get_crawler(None, {"FILES_STORE": self.tempdir}))
         for pipe_attr, settings_attr, pipe_ins_attr in self.file_cls_attr_settings_map:
             custom_value = getattr(pipe, pipe_ins_attr)
             self.assertNotEqual(custom_value, self.default_cls_settings[pipe_attr])
@@ -398,7 +403,7 @@ class FilesPipelineTestCaseCustomSettings(unittest.TestCase):
         """
         pipeline_cls = self._generate_fake_pipeline()
         settings = self._generate_fake_settings()
-        pipeline = pipeline_cls.from_settings(Settings(settings))
+        pipeline = pipeline_cls.from_crawler(get_crawler(None, settings))
         for pipe_attr, settings_attr, pipe_ins_attr in self.file_cls_attr_settings_map:
             value = getattr(pipeline, pipe_ins_attr)
             setting_value = settings.get(settings_attr)
@@ -414,8 +419,8 @@ class FilesPipelineTestCaseCustomSettings(unittest.TestCase):
         class UserDefinedFilesPipeline(FilesPipeline):
             pass
 
-        user_pipeline = UserDefinedFilesPipeline.from_settings(
-            Settings({"FILES_STORE": self.tempdir})
+        user_pipeline = UserDefinedFilesPipeline.from_crawler(
+            get_crawler(None, {"FILES_STORE": self.tempdir})
         )
         for pipe_attr, settings_attr, pipe_ins_attr in self.file_cls_attr_settings_map:
             # Values from settings for custom pipeline should be set on pipeline instance.
@@ -433,7 +438,9 @@ class FilesPipelineTestCaseCustomSettings(unittest.TestCase):
 
         prefix = UserDefinedFilesPipeline.__name__.upper()
         settings = self._generate_fake_settings(prefix=prefix)
-        user_pipeline = UserDefinedFilesPipeline.from_settings(Settings(settings))
+        user_pipeline = UserDefinedFilesPipeline.from_crawler(
+            get_crawler(None, settings)
+        )
         for pipe_attr, settings_attr, pipe_inst_attr in self.file_cls_attr_settings_map:
             # Values from settings for custom pipeline should be set on pipeline instance.
             custom_value = settings.get(prefix + "_" + settings_attr)
@@ -448,7 +455,7 @@ class FilesPipelineTestCaseCustomSettings(unittest.TestCase):
         pipeline_cls = self._generate_fake_pipeline()
         prefix = pipeline_cls.__name__.upper()
         settings = self._generate_fake_settings(prefix=prefix)
-        user_pipeline = pipeline_cls.from_settings(Settings(settings))
+        user_pipeline = pipeline_cls.from_crawler(get_crawler(None, settings))
         for (
             pipe_cls_attr,
             settings_attr,
@@ -463,8 +470,8 @@ class FilesPipelineTestCaseCustomSettings(unittest.TestCase):
             DEFAULT_FILES_RESULT_FIELD = "this"
             DEFAULT_FILES_URLS_FIELD = "that"
 
-        pipeline = UserDefinedFilesPipeline.from_settings(
-            Settings({"FILES_STORE": self.tempdir})
+        pipeline = UserDefinedFilesPipeline.from_crawler(
+            get_crawler(None, {"FILES_STORE": self.tempdir})
         )
         self.assertEqual(
             pipeline.files_result_field,
@@ -484,7 +491,7 @@ class FilesPipelineTestCaseCustomSettings(unittest.TestCase):
         class UserPipe(FilesPipeline):
             pass
 
-        pipeline_cls = UserPipe.from_settings(Settings(settings))
+        pipeline_cls = UserPipe.from_crawler(get_crawler(None, settings))
 
         for pipe_attr, settings_attr, pipe_inst_attr in self.file_cls_attr_settings_map:
             expected_value = settings.get(settings_attr)
@@ -495,8 +502,8 @@ class FilesPipelineTestCaseCustomSettings(unittest.TestCase):
             def file_path(self, request, response=None, info=None, *, item=None):
                 return Path("subdir") / Path(request.url).name
 
-        pipeline = CustomFilesPipelineWithPathLikeDir.from_settings(
-            Settings({"FILES_STORE": Path("./Temp")})
+        pipeline = CustomFilesPipelineWithPathLikeDir.from_crawler(
+            get_crawler(None, {"FILES_STORE": Path("./Temp")})
         )
         request = Request("http://example.com/image01.jpg")
         self.assertEqual(pipeline.file_path(request), Path("subdir/image01.jpg"))
@@ -507,11 +514,10 @@ class FilesPipelineTestCaseCustomSettings(unittest.TestCase):
         self.assertEqual(fs_store.basedir, str(path))
 
 
+@pytest.mark.requires_botocore
 class TestS3FilesStore(unittest.TestCase):
     @defer.inlineCallbacks
     def test_persist(self):
-        skip_if_no_boto()
-
         bucket = "mybucket"
         key = "export.csv"
         uri = f"s3://{bucket}/{key}"
@@ -557,8 +563,6 @@ class TestS3FilesStore(unittest.TestCase):
 
     @defer.inlineCallbacks
     def test_stat(self):
-        skip_if_no_boto()
-
         bucket = "mybucket"
         key = "export.csv"
         uri = f"s3://{bucket}/{key}"
@@ -630,20 +634,19 @@ class TestGCSFilesStore(unittest.TestCase):
             import google.cloud.storage  # noqa
         except ModuleNotFoundError:
             raise unittest.SkipTest("google-cloud-storage is not installed")
-        else:
-            with mock.patch("google.cloud.storage") as _:
-                with mock.patch("scrapy.pipelines.files.time") as _:
-                    uri = "gs://my_bucket/my_prefix/"
-                    store = GCSFilesStore(uri)
-                    store.bucket = mock.Mock()
-                    path = "full/my_data.txt"
-                    yield store.persist_file(
-                        path, mock.Mock(), info=None, meta=None, headers=None
-                    )
-                    yield store.stat_file(path, info=None)
-                    expected_blob_path = store.prefix + path
-                    store.bucket.blob.assert_called_with(expected_blob_path)
-                    store.bucket.get_blob.assert_called_with(expected_blob_path)
+        with mock.patch("google.cloud.storage") as _:
+            with mock.patch("scrapy.pipelines.files.time") as _:
+                uri = "gs://my_bucket/my_prefix/"
+                store = GCSFilesStore(uri)
+                store.bucket = mock.Mock()
+                path = "full/my_data.txt"
+                yield store.persist_file(
+                    path, mock.Mock(), info=None, meta=None, headers=None
+                )
+                yield store.stat_file(path, info=None)
+                expected_blob_path = store.prefix + path
+                store.bucket.blob.assert_called_with(expected_blob_path)
+                store.bucket.get_blob.assert_called_with(expected_blob_path)
 
 
 class TestFTPFileStore(unittest.TestCase):
@@ -692,5 +695,73 @@ def _prepare_request_object(item_url, flags=None):
     )
 
 
-if __name__ == "__main__":
-    unittest.main()
+# this is separate from the one in test_pipeline_media.py to specifically test FilesPipeline subclasses
+class BuildFromCrawlerTestCase(unittest.TestCase):
+    def setUp(self):
+        self.tempdir = mkdtemp()
+        self.crawler = get_crawler(None, {"FILES_STORE": self.tempdir})
+
+    def tearDown(self):
+        rmtree(self.tempdir)
+
+    def test_simple(self):
+        class Pipeline(FilesPipeline):
+            pass
+
+        with warnings.catch_warnings(record=True) as w:
+            pipe = Pipeline.from_crawler(self.crawler)
+            assert pipe.crawler == self.crawler
+            assert pipe._fingerprinter
+            self.assertEqual(len(w), 0)
+            assert pipe.store
+
+    def test_has_old_init(self):
+        class Pipeline(FilesPipeline):
+            def __init__(self, store_uri, download_func=None, settings=None):
+                super().__init__(store_uri, download_func, settings)
+                self._init_called = True
+
+        with warnings.catch_warnings(record=True) as w:
+            pipe = Pipeline.from_crawler(self.crawler)
+            assert pipe.crawler == self.crawler
+            assert pipe._fingerprinter
+            self.assertEqual(len(w), 2)
+            assert pipe._init_called
+
+    def test_has_from_settings(self):
+        class Pipeline(FilesPipeline):
+            _from_settings_called = False
+
+            @classmethod
+            def from_settings(cls, settings):
+                o = super().from_settings(settings)
+                o._from_settings_called = True
+                return o
+
+        with warnings.catch_warnings(record=True) as w:
+            pipe = Pipeline.from_crawler(self.crawler)
+            assert pipe.crawler == self.crawler
+            assert pipe._fingerprinter
+            self.assertEqual(len(w), 3)
+            assert pipe.store
+            assert pipe._from_settings_called
+
+    def test_has_from_crawler_and_init(self):
+        class Pipeline(FilesPipeline):
+            _from_crawler_called = False
+
+            @classmethod
+            def from_crawler(cls, crawler):
+                settings = crawler.settings
+                store_uri = settings["FILES_STORE"]
+                o = cls(store_uri, crawler=crawler)
+                o._from_crawler_called = True
+                return o
+
+        with warnings.catch_warnings(record=True) as w:
+            pipe = Pipeline.from_crawler(self.crawler)
+            assert pipe.crawler == self.crawler
+            assert pipe._fingerprinter
+            self.assertEqual(len(w), 0)
+            assert pipe.store
+            assert pipe._from_crawler_called
