@@ -62,7 +62,7 @@ class KeywordArgumentsSpider(MockServerSpider):
         },
     }
 
-    checks = []
+    checks: list[bool] = []
 
     def start_requests(self):
         data = {"key": "value", "number": 123, "callback": "some_callback"}
