@@ -1,5 +1,6 @@
 from twisted.internet import defer
 from twisted.trial.unittest import TestCase
+
 from scrapy.signals import request_left_downloader
 from scrapy.spiders import Spider
 from scrapy.utils.test import get_crawler
@@ -7,12 +8,11 @@ from tests.mockserver import MockServer
 
 
 class SignalCatcherSpider(Spider):
-    name = 'signal_catcher'
+    name = "signal_catcher"
 
     def __init__(self, crawler, url, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        crawler.signals.connect(self.on_request_left,
-                                signal=request_left_downloader)
+        crawler.signals.connect(self.on_request_left, signal=request_left_downloader)
         self.caught_times = 0
         self.start_urls = [url]
 
@@ -22,11 +22,10 @@ class SignalCatcherSpider(Spider):
         return spider
 
     def on_request_left(self, request, spider):
-        self.caught_times = self.caught_times + 1
+        self.caught_times += 1
 
 
 class TestCatching(TestCase):
-
     def setUp(self):
         self.mockserver = MockServer()
         self.mockserver.__enter__()
@@ -42,8 +41,7 @@ class TestCatching(TestCase):
 
     @defer.inlineCallbacks
     def test_timeout(self):
-        crawler = get_crawler(SignalCatcherSpider,
-                              {'DOWNLOAD_TIMEOUT': 0.1})
+        crawler = get_crawler(SignalCatcherSpider, {"DOWNLOAD_TIMEOUT": 0.1})
         yield crawler.crawl(self.mockserver.url("/delay?n=0.2"))
         self.assertEqual(crawler.spider.caught_times, 1)
 
@@ -56,5 +54,5 @@ class TestCatching(TestCase):
     @defer.inlineCallbacks
     def test_noconnect(self):
         crawler = get_crawler(SignalCatcherSpider)
-        yield crawler.crawl('http://thereisdefinetelynosuchdomain.com')
+        yield crawler.crawl("http://thereisdefinetelynosuchdomain.com")
         self.assertEqual(crawler.spider.caught_times, 1)
