@@ -1,6 +1,7 @@
-from typing import Dict, Optional
+from __future__ import annotations
+
 from unittest import TestCase
-from urllib.parse import urljoin, urlparse
+from urllib.parse import urljoin
 
 from testfixtures import LogCapture
 from twisted.internet import defer
@@ -9,6 +10,7 @@ from twisted.trial.unittest import TestCase as TwistedTestCase
 from scrapy.core.scheduler import BaseScheduler
 from scrapy.http import Request
 from scrapy.spiders import Spider
+from scrapy.utils.httpobj import urlparse_cached
 from scrapy.utils.request import fingerprint
 from scrapy.utils.test import get_crawler
 from tests.mockserver import MockServer
@@ -19,7 +21,7 @@ URLS = [urljoin("https://example.org", p) for p in PATHS]
 
 class MinimalScheduler:
     def __init__(self) -> None:
-        self.requests: Dict[bytes, Request] = {}
+        self.requests: dict[bytes, Request] = {}
 
     def has_pending_requests(self) -> bool:
         return bool(self.requests)
@@ -31,7 +33,7 @@ class MinimalScheduler:
             return True
         return False
 
-    def next_request(self) -> Optional[Request]:
+    def next_request(self) -> Request | None:
         if self.has_pending_requests():
             fp, request = self.requests.popitem()
             return request
@@ -57,7 +59,7 @@ class TestSpider(Spider):
         self.start_urls = map(mockserver.url, PATHS)
 
     def parse(self, response):
-        return {"path": urlparse(response.url).path}
+        return {"path": urlparse_cached(response).path}
 
 
 class InterfaceCheckMixin:
