@@ -1,5 +1,6 @@
-import collections.abc
-from typing import Optional
+from __future__ import annotations
+
+from collections.abc import AsyncIterator, Iterable
 from unittest import mock
 
 from testfixtures import LogCapture
@@ -112,11 +113,11 @@ class BaseAsyncSpiderMiddlewareTestCase(SpiderMiddlewareTestCase):
     Should work for process_spider_output and, when it's supported, process_start_requests.
     """
 
-    ITEM_TYPE: type
+    ITEM_TYPE: type | tuple
     RESULT_COUNT = 3  # to simplify checks, let everything return 3 objects
 
     @staticmethod
-    def _construct_mw_setting(*mw_classes, start_index: Optional[int] = None):
+    def _construct_mw_setting(*mw_classes, start_index: int | None = None):
         if start_index is None:
             start_index = 10
         return {i: c for c, i in enumerate(mw_classes, start=start_index)}
@@ -127,7 +128,7 @@ class BaseAsyncSpiderMiddlewareTestCase(SpiderMiddlewareTestCase):
         yield {"foo": 3}
 
     @defer.inlineCallbacks
-    def _get_middleware_result(self, *mw_classes, start_index: Optional[int] = None):
+    def _get_middleware_result(self, *mw_classes, start_index: int | None = None):
         setting = self._construct_mw_setting(*mw_classes, start_index=start_index)
         self.crawler = get_crawler(
             Spider, {"SPIDER_MIDDLEWARES_BASE": {}, "SPIDER_MIDDLEWARES": setting}
@@ -141,13 +142,13 @@ class BaseAsyncSpiderMiddlewareTestCase(SpiderMiddlewareTestCase):
 
     @defer.inlineCallbacks
     def _test_simple_base(
-        self, *mw_classes, downgrade: bool = False, start_index: Optional[int] = None
+        self, *mw_classes, downgrade: bool = False, start_index: int | None = None
     ):
         with LogCapture() as log:
             result = yield self._get_middleware_result(
                 *mw_classes, start_index=start_index
             )
-        self.assertIsInstance(result, collections.abc.Iterable)
+        self.assertIsInstance(result, Iterable)
         result_list = list(result)
         self.assertEqual(len(result_list), self.RESULT_COUNT)
         self.assertIsInstance(result_list[0], self.ITEM_TYPE)
@@ -155,13 +156,13 @@ class BaseAsyncSpiderMiddlewareTestCase(SpiderMiddlewareTestCase):
 
     @defer.inlineCallbacks
     def _test_asyncgen_base(
-        self, *mw_classes, downgrade: bool = False, start_index: Optional[int] = None
+        self, *mw_classes, downgrade: bool = False, start_index: int | None = None
     ):
         with LogCapture() as log:
             result = yield self._get_middleware_result(
                 *mw_classes, start_index=start_index
             )
-        self.assertIsInstance(result, collections.abc.AsyncIterator)
+        self.assertIsInstance(result, AsyncIterator)
         result_list = yield deferred_from_coro(collect_asyncgen(result))
         self.assertEqual(len(result_list), self.RESULT_COUNT)
         self.assertIsInstance(result_list[0], self.ITEM_TYPE)
@@ -170,8 +171,7 @@ class BaseAsyncSpiderMiddlewareTestCase(SpiderMiddlewareTestCase):
 
 class ProcessSpiderOutputSimpleMiddleware:
     def process_spider_output(self, response, result, spider):
-        for r in result:
-            yield r
+        yield from result
 
 
 class ProcessSpiderOutputAsyncGenMiddleware:
@@ -182,8 +182,7 @@ class ProcessSpiderOutputAsyncGenMiddleware:
 
 class ProcessSpiderOutputUniversalMiddleware:
     def process_spider_output(self, response, result, spider):
-        for r in result:
-            yield r
+        yield from result
 
     async def process_spider_output_async(self, response, result, spider):
         async for r in result:
@@ -324,22 +323,22 @@ class ProcessSpiderOutputInvalidResult(BaseAsyncSpiderMiddlewareTestCase):
 
 class ProcessStartRequestsSimpleMiddleware:
     def process_start_requests(self, start_requests, spider):
-        for r in start_requests:
-            yield r
+        yield from start_requests
 
 
 class ProcessStartRequestsSimple(BaseAsyncSpiderMiddlewareTestCase):
     """process_start_requests tests for simple start_requests"""
 
-    ITEM_TYPE = Request
+    ITEM_TYPE = (Request, dict)
     MW_SIMPLE = ProcessStartRequestsSimpleMiddleware
 
     def _start_requests(self):
-        for i in range(3):
+        for i in range(2):
             yield Request(f"https://example.com/{i}", dont_filter=True)
+        yield {"name": "test item"}
 
     @defer.inlineCallbacks
-    def _get_middleware_result(self, *mw_classes, start_index: Optional[int] = None):
+    def _get_middleware_result(self, *mw_classes, start_index: int | None = None):
         setting = self._construct_mw_setting(*mw_classes, start_index=start_index)
         self.crawler = get_crawler(
             Spider, {"SPIDER_MIDDLEWARES_BASE": {}, "SPIDER_MIDDLEWARES": setting}
@@ -443,7 +442,7 @@ class BuiltinMiddlewareSimpleTest(BaseAsyncSpiderMiddlewareTestCase):
     MW_UNIVERSAL = ProcessSpiderOutputUniversalMiddleware
 
     @defer.inlineCallbacks
-    def _get_middleware_result(self, *mw_classes, start_index: Optional[int] = None):
+    def _get_middleware_result(self, *mw_classes, start_index: int | None = None):
         setting = self._construct_mw_setting(*mw_classes, start_index=start_index)
         self.crawler = get_crawler(Spider, {"SPIDER_MIDDLEWARES": setting})
         self.spider = self.crawler._create_spider("foo")
