@@ -16,7 +16,7 @@ class CoreStatsExtensionTest(unittest.TestCase):
     @mock.patch("scrapy.extensions.corestats.datetime")
     def test_core_stats_default_stats_collector(self, mock_datetime):
         fixed_datetime = datetime(2019, 12, 1, 11, 38)
-        mock_datetime.utcnow = mock.Mock(return_value=fixed_datetime)
+        mock_datetime.now = mock.Mock(return_value=fixed_datetime)
         self.crawler.stats = StatsCollector(self.crawler)
         ext = CoreStats.from_crawler(self.crawler)
         ext.spider_opened(self.spider)
