@@ -1,7 +1,9 @@
 import functools
 import operator
 import platform
+import sys
 
+import pytest
 from twisted.trial import unittest
 
 from scrapy.utils.asyncgen import as_async_generator, collect_asyncgen
@@ -150,6 +152,7 @@ class BinaryIsTextTest(unittest.TestCase):
 
 
 class UtilsPythonTestCase(unittest.TestCase):
+    @pytest.mark.filterwarnings("ignore::scrapy.exceptions.ScrapyDeprecationWarning")
     def test_equal_attributes(self):
         class Obj:
             pass
@@ -175,7 +178,7 @@ class UtilsPythonTestCase(unittest.TestCase):
         self.assertTrue(equal_attributes(a, b, ["x", "y"]))
 
         a.y = 1
-        # differente attributes
+        # different attributes
         self.assertFalse(equal_attributes(a, b, ["x", "y"]))
 
         # test callable
@@ -238,16 +241,18 @@ class UtilsPythonTestCase(unittest.TestCase):
         self.assertEqual(get_func_args(str.split, stripself=True), ["sep", "maxsplit"])
         self.assertEqual(get_func_args(" ".join, stripself=True), ["iterable"])
 
-        if platform.python_implementation() == "CPython":
-            # This didn't work on older versions of CPython: https://github.com/python/cpython/issues/86951
+        if sys.version_info >= (3, 13) or platform.python_implementation() == "PyPy":
+            # the correct and correctly extracted signature
+            self.assertEqual(
+                get_func_args(operator.itemgetter(2), stripself=True), ["obj"]
+            )
+        elif platform.python_implementation() == "CPython":
+            # ["args", "kwargs"] is a correct result for the pre-3.13 incorrect function signature
+            # [] is an incorrect result on even older CPython (https://github.com/python/cpython/issues/86951)
             self.assertIn(
                 get_func_args(operator.itemgetter(2), stripself=True),
                 [[], ["args", "kwargs"]],
             )
-        elif platform.python_implementation() == "PyPy":
-            self.assertEqual(
-                get_func_args(operator.itemgetter(2), stripself=True), ["obj"]
-            )
 
     def test_without_none_values(self):
         self.assertEqual(without_none_values([1, None, 3, 4]), [1, 3, 4])
